/* ScummVM - Graphic Adventure Engine
 *
 * ScummVM is the legal property of its developers, whose names
 * are too numerous to list here. Please refer to the COPYRIGHT
 * file distributed with this source distribution.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/trunk/backends/platform/iphone/iphone_main.m $
 * $Id: iphone_main.m 40872 2009-05-24 21:53:29Z lordhoto $
 *
 */

//includes
#include <semaphore.h>

//imports
#import "switchViewController.h"
#import "iPhoneViewController.h"
#import "Gob1MenuViewController.h"
#import "menuigViewController.h"
#import "iphone_video.h"
#import "iphone_main.h"
#import "iphone_common.h"
#import "DECustomAlert.h"
#import <UIKit/UIKit.h>
#import <Foundation/NSThread.h>
#import <AudioToolbox/AudioServices.h>


//////////////////
//---GAME PARAMS
//LVL (GENERAL)
extern bool bLoadingLevel;
//LVL (SIMULATION)
extern bool bSimulation;
extern bool bFirstLoad;
extern bool bLvLLoaded;
extern bool bLvLOneLoaded; 
extern bool bSimulCode;
extern int  idLvLToLoad;
extern int  idLvLCourant;
extern int  idLvLHigh;
//GAME (GENERAL)
extern bool bHintMenuActive;
extern bool bMenuIngame;
extern bool bFromMenuIngame;
extern bool bAnimBeginning;
extern bool bMenuGob1;
extern bool bMouseModeOn;
extern bool iPhoneViewCall;
//SOUND
extern bool bSoundMusicActive;
extern int  idSoundMusic;
extern int  idSoundSFX;
// CD Audio
extern AVAudioPlayer* menuPlayer;
//VM
extern sem_t* sem;
extern bool bSem;
//DEFINES
extern void initScumm(int argc, char ** argv);
extern iPhoneViewController* g_iVC;
extern switchViewController* g_switchVC;
extern Gob1MenuViewController* g_gob1MenuVC;
//---END
////////////////

static int gArgc;
static char** gArgv;

bool   bBackground = false;
bool   bSleepMode = false;
UInt32 isiPodPlaying;

AVAudioPlayer* audioPlayer;
iPhoneMain*    _delegate;

int _GobLanguage = 0;
int _otherLanguages = -1;


////////////////////////////////////////////////////////////////////////////////////
//C++
int main(int argc, char** argv) 
{
	gArgc = argc;
	gArgv = argv;
	
    NSAutoreleasePool* autoreleasePool = [[NSAutoreleasePool alloc] init];
	
    int returnCode = UIApplicationMain(argc, argv, @"UIApplication", @"iPhoneMain");
    [autoreleasePool release];
    return returnCode;
}

//OBJ-C
@implementation iPhoneMain

- (id) init 
{
	//NSLog(@"iPhoneMain init");
	[super init];
	_window = nil;
	_view = nil;
	_delegate = self;
	
	//LANGUAGE
	NSArray* languages = [NSLocale preferredLanguages];
    NSString* preferredLang = [languages objectAtIndex:0];
	if( [preferredLang isEqualToString:@"en"] )
		_GobLanguage = 0;
	else if( [preferredLang isEqualToString:@"fr"] )
		_GobLanguage = 1;
	else 
	{
		_GobLanguage = 0;
		
		if( [preferredLang isEqualToString:@"es"] )
		{
			_otherLanguages = 0;
		}
		else if( [preferredLang isEqualToString:@"it"] )
		{
			_otherLanguages = 1;
		}
		else if( [preferredLang isEqualToString:@"de"] )
		{
			_otherLanguages = 2;
		}
	}
	
	//POPUPS
	
	bPopupSoundActive = true;
	bPopupResumeGameActive = true;
	bStopTimer = false;
	etatPopup = 0;
	
	//SAVE INFOS 
	docsDirectory = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    path = [docsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"SETTINGS.DAT"]];
    path_sound = [docsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"SETTINGS_SOUND.DAT"]];
    path_control = [docsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"SETTINGS_CONTROL.DAT"]];	
	
	/*NSFileManager* fileManager = [NSFileManager defaultManager];
	[fileManager removeItemAtPath:path error:NULL];
	[fileManager removeItemAtPath:path_sound error:NULL];*/
	
	//level path file reading
	NSString *fileString = [NSString stringWithContentsOfFile:path encoding:NSUTF8StringEncoding error:nil];
	NSArray *mArray = [fileString componentsSeparatedByString:@"\n"];
	NSString *ll = mArray.lastObject;
	
	//get the level ids
	if( ll != nil )
	{
		idLvLCourant = [[ll substringWithRange:NSMakeRange(7, 2)] intValue];
		if( idLvLCourant >= 10 )
			idLvLHigh    = [[ll substringWithRange:NSMakeRange(15, 2)] intValue];
		else
			idLvLHigh    = [[ll substringWithRange:NSMakeRange(14, 2)] intValue];		
	}
	
	//sound path file reading
	fileString = [NSString stringWithContentsOfFile:path_sound encoding:NSUTF8StringEncoding error:nil];
	mArray = [fileString componentsSeparatedByString:@"\n"];
	ll = mArray.lastObject;
	
	//get the sound values
	if( ll != nil )
	{
		idSoundMusic = [[ll substringWithRange:NSMakeRange(6, 3)] intValue];
		if( idSoundMusic >= 100 )
			idSoundSFX = [[ll substringWithRange:NSMakeRange(14, 3)] intValue];
		else if( idSoundMusic < 100 && idSoundMusic >= 10 )
			idSoundSFX = [[ll substringWithRange:NSMakeRange(13, 3)] intValue];		
		else
			idSoundSFX = [[ll substringWithRange:NSMakeRange(12, 3)] intValue];
	}
	
	//control file reading
	fileString = [NSString stringWithContentsOfFile:path_control encoding:NSUTF8StringEncoding error:nil];
	mArray = [fileString componentsSeparatedByString:@"\n"];
	ll = mArray.lastObject;
	
	//get the control mode
	if( ll != nil )
	{
		int a = [[ll substringWithRange:NSMakeRange(0, 1)] intValue];
		if( a == 1 )
			bMouseModeOn = true;
		else if( a == 0 )
			bMouseModeOn = false;
	}
	
	//NOTIFICATIONSS
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(button1Notification)  name:@"button1Notif" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(button2Notification)  name:@"button2Notif" object:nil];	

	return self;
}

NSThread* myThread;

- (void) mainLoop:(id)param
{
	//NSLog(@"iPhoneMain mainLoop");
	[[NSAutoreleasePool alloc] init];
	_view = (iPhoneView*)g_iVC.view;
	myThread = [NSThread currentThread];
	initScumm(gArgc,gArgv);
	[_view applicationSuspend];	
}

- (void) applicationWillterminate:(UIApplication *)application
{
	//NSLog(@"iPhoneMain applicationWillterminate");
	if ( g_iVC.view.superview != nil && g_iVC != nil ) 
		[(iPhoneView*)g_iVC.view applicationSuspend];	
	
	[_view release];
    [_iVC release];
    [_gob1MenuVC release];
    [_switchVC release];
    [_window release];
}

- (void) applicationWillResignActive:(UIApplication *)application
{
	//NSLog(@"iPhoneMain applicationWillResignActive");	
	//background active on iPhone 4
	if( [[UIDevice currentDevice].systemVersion floatValue] > 3.2) 
	{
		bBackground = true;
		if( g_iVC.view.superview != nil && g_iVC != nil ) 
			[(iPhoneView*)g_iVC.view applicationSuspend];	
	}
	
	//user break
	if( g_iVC.view.superview != nil && g_iVC != nil )
	{
		bSleepMode = true;

		//if we are in game, get the ingame menu
		if( !iPhoneViewCall && !bAnimBeginning && bLvLLoaded && !bSimulation && idLvLCourant != -2 && !bMenuIngame && !bFromMenuIngame && !bHintMenuActive )
		{
			[g_switchVC switchPause];
			bMenuIngame = true;
		}
	}

	//stop sounds
	if( audioPlayer != nil )
		[audioPlayer stop];
	
	if( menuPlayer != nil )
		[menuPlayer stop];
	
	//anim pause notif callback
	[[NSNotificationCenter defaultCenter] postNotificationName:@"pauseAnimNotif" object:nil];
}

- (void) applicationDidBecomeActive:(UIApplication *)application
{
	//NSLog(@"iPhoneMain applicationDidBecomeActive");	
	//background active iphone 4
	if( [[UIDevice currentDevice].systemVersion floatValue] > 3.2)
	{
		bBackground = false;
		if ( g_iVC.view.superview != nil && g_iVC != nil )
			[_view performSelectorOnMainThread:@selector(updateSurface) withObject:nil waitUntilDone: NO];
	}
	
	//resuming semaphore
	if( bSem && !bMenuIngame && !bHintMenuActive )
	{
		sem_post( sem );		
		bSem = false;
	}
	if( !bMenuIngame && !bHintMenuActive )
		bSleepMode = false;
	
	//check the iPod music
	UInt32 propertySize = sizeof(isiPodPlaying);	
	status = AudioSessionGetProperty(kAudioSessionProperty_OtherAudioIsPlaying, &propertySize, &isiPodPlaying);
	
	//if it's playing, music volume control = 0
	if( isiPodPlaying )
	{
		UInt32 sessionCategory = kAudioSessionCategory_AmbientSound;
		status = AudioSessionSetProperty(kAudioSessionProperty_AudioCategory, sizeof(sessionCategory), &sessionCategory);
		bSoundMusicActive = false;		
		idSoundMusic = 0;	
	}
	//else play the music
	else
	{
		UInt32 sessionCategory = kAudioSessionCategory_SoloAmbientSound;
		status = AudioSessionSetProperty(kAudioSessionProperty_AudioCategory, sizeof(sessionCategory), &sessionCategory); 
		bSoundMusicActive = true;
		[[NSNotificationCenter defaultCenter] postNotificationName:@"settingsMusicNotif" object:nil];
		
		if( !bMenuGob1 )
		{
			if ( g_iVC.view.superview != nil && g_iVC != nil ) 
			{
				if( audioPlayer != nil )
				{
					[audioPlayer stop];
					audioPlayer.volume = (float)((float)idSoundMusic / 256.0f);
					[audioPlayer play];	
				}
			}
		}
		else 
		{
			if( menuPlayer != nil )
			{
				[menuPlayer stop];
				menuPlayer.volume = (float)((float)idSoundMusic / 256.0f);
				menuPlayer.numberOfLoops = -1;
				if( g_gob1MenuVC.view.alpha == 1 && bSoundMusicActive )
					[menuPlayer play]; 	
			}
		}
	}
	
	if( bHintMenuActive )
		[audioPlayer pause];
	
	//callback anim resume
	[[NSNotificationCenter defaultCenter] postNotificationName:@"resumeAnimNotif" object:nil];
}

- (void) applicationDidEnterBackground:(UIApplication *)application
{
	//NSLog(@"iPhoneMain applicationDidEnterBackground");	
	//callback anim stop
	[[NSNotificationCenter defaultCenter] postNotificationName:@"stopAnimNotif" object:nil];
}

- (void) applicationWillEnterForeground:(UIApplication *)application
{
	//NSLog(@"iPhoneMain applicationWillEnterForeground");
	//get the iPod music value
	UInt32 propertySize = sizeof(isiPodPlaying);	
	status = AudioSessionGetProperty(kAudioSessionProperty_OtherAudioIsPlaying, &propertySize, &isiPodPlaying);
	NSURL *url = [NSURL fileURLWithPath:[NSString stringWithFormat:@"%@/music_menu_48.mp3", [[NSBundle mainBundle] resourcePath]]];
	NSError *err;
	
	//if already to 0, make it 192
	if( idSoundMusic == 0 )
		idSoundMusic = 192;
	
	//if no music, play it !
	if( !isiPodPlaying )
	{
		if( !bMenuGob1 )
		{
			if ( g_iVC.view.superview != nil && g_iVC != nil ) 
			{
				if( audioPlayer != nil )
				{
					[audioPlayer stop];
					audioPlayer.volume = (float)((float)idSoundMusic / 256.0f);
					[audioPlayer play];	
				}
			}
		}
		else 
		{
			if( menuPlayer != nil )
			{
				[menuPlayer stop];
				menuPlayer.volume = (float)((float)idSoundMusic / 256.0f);
				menuPlayer.numberOfLoops = -1;
				[menuPlayer play];		
			}
			else 
			{

				menuPlayer = [[AVAudioPlayer alloc] initWithContentsOfURL:url error:&err];			 
				[menuPlayer stop];
				menuPlayer.volume = (float)((float)idSoundMusic / 256.0f);
				menuPlayer.numberOfLoops = -1;
				[menuPlayer play];	
			}
		}
	}
	
	//callback anim start
	[[NSNotificationCenter defaultCenter] postNotificationName:@"startAnimNotif" object:nil];	
}

- (void) alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex 
{
	//NSLog(@"alertView didDismissWithButtonIndex");
	//if no sleep mode, release popup
	if( !bSem )
		[alertView release];
}

- (void) onPopupMusicActive
{
	//NSLog(@"iPhoneMain onPopupMusicActive");
	
	//si pas de popup ou si appuie sur bouton, on debloque et on passe à la suite
	if( !bPopupSoundActive )
	{
		bStopTimer = true;
		bPopupSoundActive = true;
		[self nextStep];
		[popupMusicTimer invalidate];
		popupMusicTimer = nil;		
	}
}

- (void) onPopupResumeGameActive
{
	//NSLog(@"iPhoneMain onPopupResumeGameActive");
	
	//si pas de popup ou si appuie sur bouton, on debloque et on passe à la suite
	if( !bPopupResumeGameActive )
	{
		bStopTimer = true;
		bPopupResumeGameActive = true;
		[self launchViews];
		[popupResumeTimer invalidate];
		popupResumeTimer = nil;
	}
}

- (void) button1
{
	//NSLog(@"saveViewController button1");	
	if( etatPopup == 1 )
	{
		//if iPod music
		if( isiPodPlaying )
		{     
			//NSLog(@"YES...");
			//NSLog(@"...AmbientSound");
			UInt32 sessionCategory = kAudioSessionCategory_AmbientSound;
			status = AudioSessionSetProperty(kAudioSessionProperty_AudioCategory, sizeof(sessionCategory), &sessionCategory);
			bSoundMusicActive = false;		
		}
		bPopupSoundActive = false;
	}
	else if( etatPopup == 2 )
	{
		//if resume game
		//NSLog(@"YES...");
		
		bLvLOneLoaded = true;
		bSimulCode = false;
		bLoadingLevel = false;
		bSimulation = true;
		bFirstLoad = true;		
		
		//as usual...
		docsDirectory = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
		path = [docsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"SETTINGS.DAT"]];
		NSString *fileString = [NSString stringWithContentsOfFile:path encoding:NSUTF8StringEncoding error:nil];
		NSArray *mArray = [fileString componentsSeparatedByString:@"\n"];
		NSString* ll = mArray.lastObject;
		
		if( ll != nil )
			idLvLToLoad = [[ll substringWithRange:NSMakeRange(7, 2)] intValue];
		
		//launch game directly, no game menu
		bMenuGob1 = false;
		bPopupResumeGameActive = false;
	}
	
	//dismiss alert !
	[alert dismissWithClickedButtonIndex:0 animated:YES]; 
}

- (void) button2
{
	//NSLog(@"saveViewController button2");	
	if( etatPopup == 1 )
	{
		//nop, iPod music not running
		//NSLog(@"NO...");
		//NSLog(@"...SoloAmbientSound");
		UInt32 sessionCategory = kAudioSessionCategory_SoloAmbientSound;
		status = AudioSessionSetProperty(kAudioSessionProperty_AudioCategory, sizeof(sessionCategory), &sessionCategory); 
		bSoundMusicActive = true;	
		bPopupSoundActive = false;
		idSoundMusic = 192;
	}
	else if( etatPopup == 2 )
	{
		//no resuming game
		//NSLog(@"NO...");
		
		bMenuGob1 = true;
		bPopupResumeGameActive = false;
	}
	
	//alert dismissed!
	[alert dismissWithClickedButtonIndex:1 animated:YES]; 
}

- (void) applicationDidFinishLaunching:(UIApplication *)application 
{
	//NSLog(@"iPhoneMain applicationDidFinishLaunching");	
	//blabla
	application.idleTimerDisabled = YES;
	CGRect  rect = [[UIScreen mainScreen] bounds];

	// hide the status bar
    [application setStatusBarStyle:UIStatusBarStyleBlackTranslucent animated:NO];
    [application setStatusBarHidden:YES animated:NO];

	_window = [[UIWindow alloc] initWithFrame:rect];
	[_window retain];

	//<TEST AUDIO SESSION
	UInt32 propertySize = sizeof(isiPodPlaying);	
	AudioSessionInitialize(NULL, NULL, NULL, self);
	status = AudioSessionGetProperty(kAudioSessionProperty_OtherAudioIsPlaying, &propertySize, &isiPodPlaying);
	//TEST AUDIO SESSION>
	
	//popup
	if( isiPodPlaying )
	{
		[application setStatusBarOrientation:UIInterfaceOrientationLandscapeRight animated:NO];

		//popup
		UIImage *backgroundImage = [UIImage imageNamed:@"popup_fond.png"];
		UIImage *img1button = [[UIImage imageNamed:@"popup_ok.png"] retain];
		UIImage *img2button = [[UIImage imageNamed:NSLocalizedString(@"PopupCancelImg", @"popup_cancel.png") ] retain];
		alert = [[DECustomAlert alloc] initWithImage:backgroundImage text:NSLocalizedString(@"iPodMusicTitle", @" iPod Music") sstext:NSLocalizedString(@"iPodMusicText", @"Would you like to keep your music in game?") button1:true img1:img1button button2:true img2:img2button];
		alert.delegate = self;
		[alert show];
		[backgroundImage release];
		[img1button release];
		[img2button release];
		
		etatPopup = 1;
		
		if( !bStopTimer )
			popupMusicTimer = [NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(onPopupMusicActive) userInfo:nil repeats:YES];
	}
	else
	{
		[application setStatusBarOrientation:UIInterfaceOrientationLandscapeRight animated:NO];
		[self nextStep];
	}
}

- (void) nextStep
{
	//NSLog(@"iPhoneMain nextStep");	
	//look if there is a resuming
	bStopTimer = false;
	
	docsDirectory = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    path = [docsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"SETTINGS.DAT"]];
	NSString *fileString = [NSString stringWithContentsOfFile:path encoding:NSUTF8StringEncoding error:nil];
	NSArray  *mArray = [fileString componentsSeparatedByString:@"\n"];
	NSString *ll = mArray.lastObject;
	int valLvL = -1;
	if( ll != nil )
	{
		 valLvL = [[ll substringWithRange:NSMakeRange(7, 2)] intValue];
	}
	
	NSFileManager* fileManager = [NSFileManager defaultManager];
	
	if( [fileManager fileExistsAtPath:path] && valLvL != 1 && valLvL != -1 ) //on demande pas si lvl courant = niveau 1 (pas de simul !)
	{
		//popup
		UIImage *backgroundImage = [UIImage imageNamed:@"popup_fond.png"];
		UIImage *img1button = [[UIImage imageNamed:@"popup_ok.png"] retain];
		UIImage *img2button = [[UIImage imageNamed:NSLocalizedString(@"PopupCancelImg", @"popup_cancel.png") ] retain];
		alert = nil;
		alert = [[DECustomAlert alloc] initWithImage:backgroundImage text:NSLocalizedString(@"ResumeGameTitle", @"Resume Game") sstext:NSLocalizedString(@"ResumeGameText",@"Do you want to resume the game at the last level you played?") button1:true img1:img1button button2:true img2:img2button];
		alert.delegate = self;
		[alert show];
		[backgroundImage release];
		[img1button release];
		[img2button release];
		
		etatPopup = 2;
		
		if( !bStopTimer )
			popupResumeTimer = [NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(onPopupResumeGameActive) userInfo:nil repeats:YES];
	}
	else
	{
		[self launchViews];
	}
}

- (void) launchViews
{
	//NSLog(@"iPhoneMain launchViews");	
	//load everything, views, sound ...
	//[[UIDevice currentDevice] beginGeneratingDeviceOrientationNotifications];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillterminate:) name:UIApplicationWillTerminateNotification object:nil];
	
	// Emulated CD support through MP3
	NSURL *url = [NSURL fileURLWithPath:[NSString stringWithFormat:@"%@/track1_24.mp3", [[NSBundle mainBundle] resourcePath]]];
	NSError *error;
	audioPlayer = [[AVAudioPlayer alloc] initWithContentsOfURL:url error:&error];
	audioPlayer.numberOfLoops = -1;
	
	//go to SWITCH VIEW 
	_switchVC = [[switchViewController alloc] initWithNibName:@"switchViewController" bundle:[NSBundle mainBundle]];
	//_view = _switchVC.view; 
	_switchVC.view.multipleTouchEnabled = NO;

	[_window addSubview:_switchVC.view];
	[_window makeKeyAndVisible];
}

- (void) rotationNotification
{
	//NSLog(@"iPhoneMain rotationNotification");		
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(didRotate:)
												 name:@"UIDeviceOrientationDidChangeNotification" object:nil];
}

- (void) didRotate:(NSNotification *)notification
{
	//NSLog(@"iPhoneMain didRotate");			
	UIDeviceOrientation screenOrientation = [[UIDevice currentDevice] orientation]; 
	
	if ( g_iVC.view.superview != nil && g_iVC != nil ) 
		[_view deviceOrientationChanged: screenOrientation];
}

//NOTIFICATIONS
- (void) button1Notification
{
	//NSLog(@"saveViewController button1Notification");	
	[self performSelectorOnMainThread:@selector(button1) withObject: nil waitUntilDone:NO];
}

- (void) button2Notification
{
	//NSLog(@"saveViewController button2Notification");	
	[self performSelectorOnMainThread:@selector(button2) withObject: nil waitUntilDone:NO];
}

//ACCESSORS
- (UIWindow*) getWindow 
{
	//NSLog(@"iPhoneMain getWindow");		
	return _window;
}

- (iPhoneView*) getView
{
	//NSLog(@"iPhoneMain getView");	
	return _view;
}

- (void) setView:(iPhoneView*)iView
{
	//NSLog(@"iPhoneMain setView");		
	_view = iView;
}

@end

