/* ScummVM - Graphic Adventure Engine
 *
 * ScummVM is the legal property of its developers, whose names
 * are too numerous to list here. Please refer to the COPYRIGHT
 * file distributed with this source distribution.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/trunk/backends/platform/iphone/iphone_video.m $
 * $Id: iphone_video.m 44495 2009-09-30 16:16:53Z fingolfin $
 *
 */

//INCLUDES
#include <semaphore.h>
#include "system.h"
#include <fstream>
#include <math.h>
#include <unistd.h>
#include <sys/time.h>
#include <vector>
#include <sys/types.h>
#include <sys/sysctl.h>

#import <engines/engine.h>
#import <engines/igob/gob.h>
#import <engines/igob/global.h>
#import <engines/igob/goblin.h>
#import <engines/igob/script.h>
#import <engines/igob/hotspots.h>
#import <engines/igob/game.h>

#include "iphone_video.h"
#include "osys_main.h"
#import "menuigViewController.h"
#import "switchViewController.h"
#import "DECustomAlert.h"

//---VIDEO VIEW PARAMS
static char* _textureBuffer = 0;
static int _width = 0;
static int _height = 0;
static int _textureWidth = 0;
static int _textureHeight = 0;
static int _needsScreenUpdate = 0;
static CGRect _screenRect;
static iPhoneView *sharedInstance = nil;
//---END

//////////////////
//---GAME PARAMS
//LVL (GENERAL)
extern const char* niveau;
extern bool bLoadingLevel;
extern bool bLvLLoaded;
extern bool bGoHotspot;
extern bool iPhoneViewCall;
extern int idLvLCourant;
extern int idLvLHigh;
extern int cptSound;
//LVL (SIMULATION)
extern bool bSimulation;
//GAME (GENERAL)
extern bool bGobPicker;
extern bool bPopup;
extern bool bGameOverOn;
extern bool bMenuIngame;
extern bool bHintMenuActive;
extern bool bFromMenuIngame;
extern bool bAnimBeginning;
extern bool bMenuGob1;
UIButton* externMenuButton;
UIButton* externHintButton;
CGPoint dist;
CGPoint m;
int numGobliiin = 0;
bool bTutoOn = false;
bool bMouseModeOn = false;
bool bMouseCursorActive = true;
//SOUND
extern bool bSoundMusicActive;
extern bool bSetSounds;
extern int idSoundMusic;
extern int idSoundSFX;
extern AVAudioPlayer* audioPlayer;
//DEFINES
extern switchViewController* g_switchVC;
extern menuigViewController* g_menuigVC;
extern OSystem* g_system;
extern std::vector< Gob::Goblin::Gob_Object* > _HotspotObj;
extern const char* m_lvl[22];
extern int16 myHackSpotL;
extern int16 myHackSpotT;
extern bool bBackground;
extern bool bSleepMode;
extern bool bSem;
extern sem_t* sem;
extern int _GobLanguage;
//---END
////////////////

bool bProtecOFF = false;
NSString* _plat;

//C-declarations
uint getSizeNextPOT(uint size);
//END

@implementation iPhoneView

+ (Class) layerClass
{
	return [CAEAGLLayer class];
}

- (id)initWithFrame:(struct CGRect)frame
{
	//NSLog(@"iPhoneView initWithFrame");
	[super initWithFrame: frame];
	_fullWidth = frame.size.width;
	_fullHeight = frame.size.height;
	_screenLayer = nil;
	
	sharedInstance = self;
	
	_lock = [NSLock new];
	_context = nil;
	_screenTexture = 0;
	
	return self;
}

- (void)awakeFromNib
{
	//NSLog(@"iPhoneView awakeFromNib");	
	_fullWidth = self.frame.size.width;
	_fullHeight = self.frame.size.height;
	_screenLayer = nil;
	
	sharedInstance = self;
	
	_lock = [NSLock new];
	_context = nil;
	_screenTexture = 0;
	
	//init hotspots & gestion of them (anim, fade...)
	for( int i = 0; i < 20; ++i )
	{
		hotspotDisplayed[i] = false;
		hotspotUsed[i] = false;
		alphaDisplayed[i] = false;
	}
	iconsDisplayed = false;
	alphaButtonsDisplayed = false;
	numGobliiin = 0;
	
	//display buttons - use Action icon
	useIcon = [[UIButton buttonWithType:UIButtonTypeCustom] retain];
	useIcon.frame = CGRectMake( 0, 0, 0, 0 );
	[useIcon setImage:[UIImage imageNamed:@"Marron-poing3.png"] forState:UIControlStateNormal];
	[useIcon addTarget:self action:@selector(useIconClicked:) forControlEvents:UIControlEventTouchDown];
	useIcon.exclusiveTouch = YES;
	
	//display buttons - pick Action icon
	pickIcon = [[UIButton buttonWithType:UIButtonTypeCustom] retain];
	pickIcon.frame = CGRectMake( 0, 0, 0, 0 );
	[pickIcon setImage:[UIImage imageNamed:@"Marron-Main4.png"] forState:UIControlStateNormal];
	[pickIcon addTarget:self action:@selector(pickIconClicked:) forControlEvents:UIControlEventTouchDown];
	pickIcon.exclusiveTouch = YES;
	
	//display buttons - step Action icon
	stepIcon = [[UIButton buttonWithType:UIButtonTypeCustom] retain];
	stepIcon.frame = CGRectMake( 0, 0, 0, 0 );
	[stepIcon setImage:[UIImage imageNamed:@"Marron-Pied3.png"] forState:UIControlStateNormal];
	[stepIcon addTarget:self action:@selector(stepIconClicked:) forControlEvents:UIControlEventTouchDown];
	stepIcon.exclusiveTouch = YES;
	
	//display buttons - cancel Action icon
	cancelIcon = [[UIButton buttonWithType:UIButtonTypeCustom] retain];
	cancelIcon.frame = CGRectMake( 0, 0, 0, 0 );
	[cancelIcon setImage:[UIImage imageNamed:@"Marron-Croix3.png"] forState:UIControlStateNormal];
	[cancelIcon addTarget:self action:@selector(cancelIconClicked:) forControlEvents:UIControlEventTouchDown];
	cancelIcon.exclusiveTouch = YES;
		
	//ingame menu buttons
	skipButton.hidden = YES;
	skipButton.exclusiveTouch = YES;
	choiceButton.hidden = YES;
	choiceButton.exclusiveTouch = YES;
	hintButton.hidden = YES;
	hintButton.exclusiveTouch = YES;
	menuButton.hidden = YES;
	menuButton.exclusiveTouch = YES;
	barreInventaire.hidden = YES;
	
	externMenuButton = menuButton;
	externHintButton = hintButton;
	
	//GO notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(goNotification)  name:@"goNotif" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(goMsgNotification)  name:@"goMsgNotif" object:nil];
	//game over notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(gameOverNotification)  name:@"gameOverNotif" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(gameOverOffNotification)  name:@"gameOverOffNotif" object:nil];	
	//touch notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(levelBeginNotification)  name:@"levelBeginNotif" object:nil];
	//noInteraction notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(noInteractionNotification)  name:@"noInteractionNotif" object:nil];
	//clearAll notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(clearAllNotification)  name:@"clearAllNotif" object:nil];	
	//hide buttons notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(hideButtonsNotification)  name:@"hideButtonsNotif" object:nil];	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(skipButtonOnNotification)  name:@"skipButtonOnNotif" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(skipButtonOffNotification)  name:@"skipButtonOffNotif" object:nil];
	//set sounds notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(setSoundsNotification)  name:@"setSoundsNotif" object:nil];	
	//level notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(lvlLoadedNotification)  name:@"lvlLoadedNotif" object:nil];
	//loading notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(loadingScreenOnNotification)  name:@"loadingScreenOnNotif" object:nil];	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(loadingScreenOffNotification)  name:@"loadingScreenOff" object:nil];		
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(loadingOffAnimBeginningNotification)  name:@"loadingOffAnimBeginningNotif" object:nil];
	//end game notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(endGameNotification)  name:@"endGameNotif" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(endAnimEndGameNotification)  name:@"endAnimEndGameNotif" object:nil];
	//control mode notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(cursorModeNotification)  name:@"cursorModeNotif" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(touchModeNotification)  name:@"touchModeNotif" object:nil];
	//maj gobliiin
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(majGobliiinNotification)  name:@"majGobliiinNotif" object:nil];	
	//alert
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(iPhoneViewAlertnotification)  name:@"iPhoneViewAlertNotif" object:nil];
	//hack sound level 5
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(hackSoundLevel5Notification)  name:@"hackSoundLevel5Notif" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(endTutoAgainNotification)  name:@"endTutoAgainNotif" object:nil];	

	mEtatAction = 0;
	stepPosX = 0.0f;
	stepPosY = 0.0f;
	bStepIconOn = false;
}

-(void) writeSettingsLvL
{
	NSLog(@"iPhoneView writeSettingsLvL");		
    std::fstream settingsFile;
	
	docsDirectory = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    path = [docsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"SETTINGS.DAT"]];
	
    settingsFile.open([path UTF8String], std::fstream::out | std::fstream::app);
    if(!settingsFile.is_open())
    {
		NSLog(@"Can't create/open file %@", path);
    }
	
    NSString* infosTabSeparated = [[NSString alloc] initWithFormat:@"niveau %i prog %i   \t", idLvLCourant, idLvLHigh];
    int buffLength = [infosTabSeparated length];
    char* buffer = new char[buffLength];
    [infosTabSeparated getBytes:(void*)buffer maxLength:buffLength usedLength:NULL encoding:NSUTF8StringEncoding options:NSStringEncodingConversionAllowLossy range:NSMakeRange(0, buffLength) remainingRange:NULL];
    settingsFile.write (buffer, buffLength);
    settingsFile.close();
    [infosTabSeparated release];
    delete[] buffer;
}

-(void) writeSettingsControl
{
	NSLog(@"iPhoneView writeSettingsControl");		
    std::fstream controlFile;
	
	docsDirectory = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    path_control = [docsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"SETTINGS_CONTROL.DAT"]];
	
    controlFile.open([path_control UTF8String], std::fstream::out | std::fstream::app);
    if(!controlFile.is_open())
    {
        NSLog(@"Can't create/open file %@", path);
    }
	int a = -1;
	if( bMouseModeOn )
		a = 1;
	else
		a = 0;
    NSString* infosTabSeparated = [[NSString alloc] initWithFormat:@"%i   \t", a];
    int buffLength = [infosTabSeparated length];
    char* buffer = new char[buffLength];
    [infosTabSeparated getBytes:(void*)buffer maxLength:buffLength usedLength:NULL encoding:NSUTF8StringEncoding options:NSStringEncodingConversionAllowLossy range:NSMakeRange(0, buffLength) remainingRange:NULL];
    controlFile.write (buffer, buffLength);
    controlFile.close();
    [infosTabSeparated release];
    delete[] buffer;
}

-(void) writeSettingsSound
{
	NSLog(@"iPhoneView writeSettingsSound");		
    std::fstream settingsFile;
	
	docsDirectory = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];	
    path_sound = [docsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"SETTINGS_SOUND.DAT"]];
	
    settingsFile.open([path_sound UTF8String], std::fstream::out | std::fstream::app);
    if(!settingsFile.is_open())
    {
        NSLog(@"Can't create/open file %@", path);
    }
	
	idSoundSFX = g_system->getMixer()->getVolumeForSoundType(Audio::Mixer::kSFXSoundType);	
	
    NSString* infosTabSeparated;
	if( !bSoundMusicActive )
	{
		infosTabSeparated = [[NSString alloc] initWithFormat:@"music %i sfx %i    \t", 192, idSoundSFX];
	}		
	else 
		infosTabSeparated = [[NSString alloc] initWithFormat:@"music %i sfx %i    \t", idSoundMusic, idSoundSFX];		

    int buffLength = [infosTabSeparated length];
    char* buffer = new char[buffLength];
    [infosTabSeparated getBytes:(void*)buffer maxLength:buffLength usedLength:NULL encoding:NSUTF8StringEncoding options:NSStringEncodingConversionAllowLossy range:NSMakeRange(0, buffLength) remainingRange:NULL];
    settingsFile.write (buffer, buffLength);
    settingsFile.close();
    [infosTabSeparated release];
    delete[] buffer;
}

- (void)applicationSuspend 
{
	//NSLog(@"iPhoneView applicationSuspend");			
	docsDirectory = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    path = [docsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"SETTINGS.DAT"]];
    path_sound = [docsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"SETTINGS_SOUND.DAT"]];
    path_control = [docsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"SETTINGS_CONTROL.DAT"]];	
	
	NSFileManager* fileManager = [NSFileManager defaultManager];
	[fileManager removeItemAtPath:path error:NULL];
	[fileManager removeItemAtPath:path_sound error:NULL];
	[fileManager removeItemAtPath:path_control error:NULL];
	
	[self writeSettingsLvL];
	[self writeSettingsSound];
	[self writeSettingsControl];
	
	NSData* someData = [[NSData alloc] initWithContentsOfFile:[docsDirectory stringByAppendingPathComponent:@"SETTINGS.DAT"]];
	//NSLog(@"%@",[[[NSString alloc] initWithData:someData encoding:NSUTF8StringEncoding] autorelease]);
	
	NSData* someData2 = [[NSData alloc] initWithContentsOfFile:[docsDirectory stringByAppendingPathComponent:@"SETTINGS_SOUND.DAT"]];
	//NSLog(@"%@",[[[NSString alloc] initWithData:someData2 encoding:NSUTF8StringEncoding] autorelease]);
	
	NSData* someData3 = [[NSData alloc] initWithContentsOfFile:[docsDirectory stringByAppendingPathComponent:@"SETTINGS_CONTROL.DAT"]];
	//NSLog(@"%@",[[[NSString alloc] initWithData:someData3 encoding:NSUTF8StringEncoding] autorelease]);
}

///----NOTIFICATIONS
-(void)iPhoneViewAlertnotification
{
	[self performSelectorOnMainThread:@selector(iPhoneViewAlert) withObject: nil waitUntilDone:NO];
}

-(void)majGobliiinNotification
{
	[self performSelectorOnMainThread:@selector(majGobliiin) withObject: nil waitUntilDone:NO];
}

-(void)loadingScreenOnNotification
{
	[self performSelectorOnMainThread:@selector(loadingScreenOn) withObject: nil waitUntilDone:NO];
}

-(void)loadingScreenOffNotification
{
	[self performSelectorOnMainThread:@selector(loadingScreenOff) withObject: nil waitUntilDone:NO];
}

-(void)loadingOffAnimBeginningNotification
{
	[self performSelectorOnMainThread:@selector(loadingOffAnimBeginning) withObject: nil waitUntilDone:NO];
}

-(void)noInteractionNotification
{
	[self performSelectorOnMainThread:@selector(noInteraction) withObject: nil waitUntilDone:NO];
}

-(void)setSoundsNotification
{
	[self performSelectorOnMainThread:@selector(setSounds) withObject: nil waitUntilDone:NO];
}

-(void)gameOverOffNotification
{
	[self performSelectorOnMainThread:@selector(gameOverOff) withObject: nil waitUntilDone:NO];
}

-(void)gameOverNotification
{
	[self performSelectorOnMainThread:@selector(gameOver) withObject: nil waitUntilDone:NO];
}

-(void)goNotification
{
	[self performSelectorOnMainThread:@selector(launchGoSimul) withObject: nil waitUntilDone:NO];
}

-(void)goMsgNotification
{
	[self performSelectorOnMainThread:@selector(displayGoMsg) withObject: nil waitUntilDone:NO];
}

-(void)clearAllNotification
{
	[self performSelectorOnMainThread:@selector(clearAll) withObject: nil waitUntilDone:NO];
}

-(void)levelBeginNotification
{
	[self performSelectorOnMainThread:@selector(levelBegin) withObject: nil waitUntilDone:NO];
}

-(void)touchModeNotification
{
	[self performSelectorOnMainThread:@selector(touchMode) withObject: nil waitUntilDone:NO];		
}

-(void)cursorModeNotification
{
	[self performSelectorOnMainThread:@selector(cursorMode) withObject: nil waitUntilDone:NO];		
}

-(void)lvlLoadedNotification
{
	[self performSelectorOnMainThread:@selector(levelLoaded) withObject: nil waitUntilDone:NO];	
}

-(void)hideButtonsNotification
{
	[self performSelectorOnMainThread:@selector(hideButtons) withObject: nil waitUntilDone:NO];	
}

-(void)skipButtonOnNotification
{
	[self performSelectorOnMainThread:@selector(skipButtonOn) withObject: nil waitUntilDone:NO];	
}

-(void)skipButtonOffNotification
{
	[self performSelectorOnMainThread:@selector(skipButtonOff) withObject: nil waitUntilDone:NO];	
}

-(void)endAnimEndGameNotification
{
	[self performSelectorOnMainThread:@selector(endAnimEndGame) withObject: nil waitUntilDone:NO];	
}

-(void)endGameNotification
{
	[self performSelectorOnMainThread:@selector(endGame) withObject: nil waitUntilDone:NO];	
}

-(void)hackSoundLevel5Notification
{
	[self performSelectorOnMainThread:@selector(hackSoundLevel5) withObject: nil waitUntilDone:NO];	
}

-(void)endTutoAgainNotification
{
	[self performSelectorOnMainThread:@selector(endTutoAgain) withObject: nil waitUntilDone:NO];	
}
//---END NOTIFICATIONS

-(void)doHackSoundLvL5
{
	//NSLog(@"hack sound Lvl 5");
	[audioPlayer stop];
}

-(void)hackSoundLevel5
{
	[NSTimer scheduledTimerWithTimeInterval:22 target:self selector:@selector(doHackSoundLvL5) userInfo:nil repeats:NO];	
}

-(void)loadingScreenOn
{
	//NSLog(@"iPhoneView loadingScreenOn");	
	bLoadingImage = true;
	if( bGameOverOn )
	{
		[gameoverV removeFromSuperview];
		[gameoverV release];
		gameoverV = nil;
	}
	
	loadingV.hidden = NO;
	loadingV.alpha = 0;
	
	if( loadingV.superview == nil )
	{
		[self addSubview:loadingV];
	}
	[self insertSubview:activityIndicator aboveSubview:loadingV];

	[UIView beginAnimations:nil context:self]; 
	{
		[UIView setAnimationDuration:0.05];
		[UIView setAnimationDelegate:self];
		[UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
		loadingV.alpha = 1.0;
	}
	[UIView commitAnimations];
	
	[self setUserInteractionEnabled:NO];
	[activityIndicator startAnimating];
	activityIndicator.hidden = NO;	
	
	if( audioPlayer != nil )
		audioPlayer.volume = (float)((float)0 / 256.0f);
	
	[[NSNotificationCenter defaultCenter] postNotificationName:@"disableButtonsNotif" object:nil];	
}

-(void)loadingScreenOff
{
	//NSLog(@"iPhoneView loadingScreenOff");	
	bLoadingImage = false;
	if( loadingV != nil )
	{
		[activityIndicator stopAnimating];
		activityIndicator.hidden = YES;
		[UIView beginAnimations:nil context:self]; 
		{
			[UIView setAnimationDuration:1];
			[UIView setAnimationDelegate:self];
			[UIView setAnimationDidStopSelector:@selector(animationFinished:finished:context:)];
		}
		[UIView commitAnimations];
	}
	
	if( audioPlayer != nil )
		audioPlayer.volume = (float)((float)idSoundMusic / 256.0f);	
}

//anim finished
- (void)animationFinished:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context 
{	
	[self setUserInteractionEnabled:YES];	
	[loadingV removeFromSuperview];
	loadingV.hidden = YES;	
}

-(void)lOAB
{
	[[NSNotificationCenter defaultCenter] postNotificationName:@"skipButtonOnNotif" object:nil];
	[[NSNotificationCenter defaultCenter] postNotificationName:@"loadingScreenOff" object:nil];		
}

-(void)loadingOffAnimBeginning
{
	//NSLog(@"iPhoneView loadingOffAnimBeginning");		
	[NSTimer scheduledTimerWithTimeInterval:2 target:self selector:@selector(lOAB) userInfo:nil repeats:NO];
}

-(void)noInteraction
{
	//NSLog(@"iPhoneView noInteraction");	
	
	//don't touch
	[self setUserInteractionEnabled:NO];
}

-(void)setSounds
{
	//NSLog(@"iPhoneView setSounds");		
	if( !bLoadingImage )
	{
		g_system->getMixer()->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, idSoundSFX);
		audioPlayer.volume = (float)((float)idSoundMusic / 256.0f);
		bSetSounds = false;
	}
}

-(void)gameOverOff
{
	[gameoverV removeFromSuperview];
}

-(void)gameOver
{
	NSLog(@"iPhoneView gameOver");
	gameoverV = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, 480, 320)];
	UIImage *img = [[UIImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Gameover2" ofType:@"png"]];
	[gameoverV setImage:img];
	[img release];
	[self addSubview:gameoverV];
	
	//orientation	
	if( [[UIDevice currentDevice] orientation] == UIDeviceOrientationLandscapeLeft && [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationLandscapeLeft )
		[[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeLeft animated:NO];
	else if( [[UIDevice currentDevice] orientation] == UIDeviceOrientationLandscapeRight && [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationLandscapeRight )
		[[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeRight animated:NO];
	/*else
	 {
	 if( [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationPortrait || [[UIDevice currentDevice] orientation] == UIDeviceOrientationPortrait )
	 [[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeRight animated:NO];
	 }*/
	
	//popup
	UIImage *backgroundImage = [UIImage imageNamed:@"popup_fond.png"];
	UIImage *img1button = [[UIImage imageNamed:@"popup_ok_big.png"] retain];	
	alert = nil;
	alert = [[DECustomAlert alloc] initWithImage:backgroundImage text:NSLocalizedString(@"GameOverTitle", @"Game Over!") sstext:NSLocalizedString(@"GameOverText", @"Game Over! Try again!") button1:true img1:img1button button2:false img2:nil];
	alert.delegate = self;
	[alert show];
	[backgroundImage release];
	[img1button release];
	iPhoneViewCall = true;
	
	NSURL *url = [NSURL fileURLWithPath:[NSString stringWithFormat:@"%@/blop.mp3", [[NSBundle mainBundle] resourcePath]]];
	NSError *error;
	popupSound = [[AVAudioPlayer alloc] initWithContentsOfURL:url error:&error];
	popupSound.volume = (float)((float)idSoundSFX / 256.0f);
	popupSound.numberOfLoops = 0;
	[popupSound play];
	
	menuButton.hidden = YES;
	choiceButton.hidden = YES;
	hintButton.hidden = YES;	
	cursorGobliiin.hidden = YES;
	
	//[g_switchVC switchPause];
	//bMenuIngame = true;
	
	//v1.1.0 fix crash game over:
	if( bMenuIngame )
	{
		[g_menuigVC.view removeFromSuperview];
		
		bMenuIngame = false;
		bSleepMode = false;
		if( bSem && !bTutoOn )
		{
			sem_post( sem );		
			bSem = false;
		}
	}
	//
}

-(void)launchGoSimul
{
	NSLog(@"iPhoneView launchGoSimul");

	//etat action
	if( mEtatAction != 0 )
		[self resetAction:0 coordY:0];
	
	//clear all attributes
	[self clearButtons];
	
	if( bMouseModeOn )
	{
		if( cursorGobliiin != nil && cursorGobliiin.superview != nil )
		{
			[cursorGobliiin removeFromSuperview];
			cursorGobliiin = nil;
		}
	}
	
	//simul
	float xTouch = myHackSpotL + 5;
	float yTouch = myHackSpotT + 5;
	g_system->warpMouse( xTouch, yTouch );
	
	[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kTempo], @"type",
					[NSNumber numberWithFloat:xTouch], @"x",
					[NSNumber numberWithFloat:yTouch], @"y",nil]];	
	[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
					[NSNumber numberWithFloat:xTouch], @"x",
					[NSNumber numberWithFloat:yTouch], @"y",nil]];	
	[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kGo], @"type",
					[NSNumber numberWithFloat:xTouch], @"x",
					[NSNumber numberWithFloat:yTouch], @"y",nil]];
	
	//don't touch
	[self setUserInteractionEnabled:NO];
}

-(void)displayGoMsg
{
	NSLog(@"iPhoneView displayGoMsg");
	
	//clear everything 
	[self clearButtons];
	
	if( bMouseModeOn )
	{
		if( cursorGobliiin != nil && cursorGobliiin.superview != nil )
		{
			[cursorGobliiin removeFromSuperview];
			cursorGobliiin = nil;
		}
	}
	
	if( rondV != nil && rondV.superview != nil )
	{
		[rondV removeFromSuperview];
		rondV = nil;
	}
	
	for( int i = 0; i < _HotspotObj.size(); ++i )
	{
		if( hotspotImgView[i] != nil && hotspotImgView[i].superview != nil )
		{
			[hotspotImgView[i] removeFromSuperview];
			hotspotImgView[i] = nil;
		}
	}
	
	//orientation
	if( [[UIDevice currentDevice] orientation] == UIDeviceOrientationLandscapeLeft && [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationLandscapeLeft )
		[[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeLeft animated:NO];
	else if( [[UIDevice currentDevice] orientation] == UIDeviceOrientationLandscapeRight && [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationLandscapeRight )
		[[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeRight animated:NO];
	/*else
	 {
	 if( [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationPortrait || [[UIDevice currentDevice] orientation] == UIDeviceOrientationPortrait )
	 [[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeRight animated:NO];
	 }*/
	
	//popup
	UIImage *backgroundImage = [UIImage imageNamed:@"popup_fond.png"];
	UIImage *img1button = [[UIImage imageNamed:@"popup_ok_big.png"] retain];
	alert = nil;
	alert = [[DECustomAlert alloc] initWithImage:backgroundImage text:NSLocalizedString(@"LevelClearedTitle", @"Level Cleared") sstext:NSLocalizedString(@"LevelClearedText", @"Congratulations! Next level is this way!") button1:true img1:img1button button2:false img2:nil];
	alert.delegate = self;
	[alert show];
	[backgroundImage release];
	[img1button release];
	iPhoneViewCall = true;
	
	NSURL *url = [NSURL fileURLWithPath:[NSString stringWithFormat:@"%@/blop.mp3", [[NSBundle mainBundle] resourcePath]]];
	NSError *error;
	popupSound = [[AVAudioPlayer alloc] initWithContentsOfURL:url error:&error];
	popupSound.volume = (float)((float)idSoundSFX / 256.0f);
	popupSound.numberOfLoops = 0;
	[popupSound play];
}

-(void)menuDisappear
{
	[g_menuigVC.view removeFromSuperview];
}

- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex 
{
	NSLog(@"didDismissWithButtonIndex");
	if( !bSleepMode )
		[alertView release];	
}

-(void)iPhoneViewAlert
{
	if( !bPopup )
	{     
		//don't touch
		[self setUserInteractionEnabled:YES];
		
		NSLog(@"in game over %d,%d",myHackSpotL, myHackSpotT);
		float xTouch = 136 + 5;
		float yTouch = 52 + 5;
		g_system->warpMouse( xTouch, yTouch );
		
		[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
						[NSNumber numberWithFloat:xTouch], @"x",
						[NSNumber numberWithFloat:yTouch], @"y",nil]];	
		
		[[NSNotificationCenter defaultCenter] postNotificationName:@"loadingScreenOnNotif" object:nil];	
	}
	[alert dismissWithClickedButtonIndex:0 animated:YES];
	iPhoneViewCall = false;
}

-(void)clearAll
{
	[self clearButtons];
}

- (NSString *) platform 
{
	size_t size;
	sysctlbyname("hw.machine", NULL, &size, NULL, 0);
	char *machine = (char*)malloc(size);
	sysctlbyname("hw.machine", machine, &size, NULL, 0);
	NSString *platformCode = [[NSString stringWithCString:machine encoding:NSUTF8StringEncoding] retain];
	free(machine);
	return platformCode;
}

-(void)levelBegin
{
	NSLog(@"iPhoneView levelBegin");	
	_plat = [self platform];
	
	[self setUserInteractionEnabled:YES];
	bGoHotspot = false;
	bLvLLoaded = false;
	bAnimBeginning = false;
	skipButton.hidden = YES;
	bMenuIngame = false;
	bFromMenuIngame = false;
	
	externMenuButton.enabled = YES;
	externHintButton.enabled = YES;
	
	//num gobliiin
	numGobliiin = 0;
	
	//for those who need it
	cptSound = 0;
	
	//mEtatAction
	mEtatAction = 0;
	
	if( !bSimulation ) //si on simule pas un niveau
	{
		//SAVE INFOS LVL - DEBUT DU NIVEAU
		docsDirectory = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
		path = [docsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"SETTINGS.DAT"]];
		path_sound = [docsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"SETTINGS_SOUND.DAT"]];
		path_control = [docsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"SETTINGS_CONTROL.DAT"]];
		
		NSFileManager* fileManager = [NSFileManager defaultManager];
		[fileManager removeItemAtPath:path error:NULL];
		[fileManager removeItemAtPath:path_sound error:NULL];
		[fileManager removeItemAtPath:path_control error:NULL];	
		
		[self writeSettingsLvL];
		[self writeSettingsSound];		
		[self writeSettingsControl];
		
		NSData* someData = [[NSData alloc] initWithContentsOfFile:[docsDirectory stringByAppendingPathComponent:@"SETTINGS.DAT"]];
		NSLog(@"%@",[[[NSString alloc] initWithData:someData encoding:NSUTF8StringEncoding] autorelease]);
		
		NSData* someData2 = [[NSData alloc] initWithContentsOfFile:[docsDirectory stringByAppendingPathComponent:@"SETTINGS_CONTROL.DAT"]];
		NSLog(@"%@",[[[NSString alloc] initWithData:someData2 encoding:NSUTF8StringEncoding] autorelease]);
		
		NSData* someData3 = [[NSData alloc] initWithContentsOfFile:[docsDirectory stringByAppendingPathComponent:@"SETTINGS_SOUND.DAT"]];
		NSLog(@"%@",[[[NSString alloc] initWithData:someData3 encoding:NSUTF8StringEncoding] autorelease]);
	}
	
	if( bGameOverOn )
	{
		bGameOverOn = false;		
		bLvLLoaded = true;
		[[NSNotificationCenter defaultCenter] postNotificationName:@"lvlLoadedNotif" object:nil];
	}
	
	//orientation
	[self deviceOrientationChanged:[[UIDevice currentDevice] orientation]];
}

//anim alpha
-(void) beginAlphaButtons
{
	//NSLog(@"beginAlphaButtons");
	if( alphaButtonsDisplayed )
	{
		for( int i = 0; i < _HotspotObj.size(); ++i )
			hotspotImgView[i].alpha = 1;
		
		[UIView beginAnimations:nil context:self]; 
		{
			[UIView setAnimationDuration:0.35];
			[UIView setAnimationDelegate:self];
			[UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
			[UIView setAnimationDidStopSelector:@selector(alphaButtonsFinished:finished:context:)];
			for( int i = 0; i < _HotspotObj.size(); ++i )
				hotspotImgView[i].alpha = 0.25;
		}
		[UIView commitAnimations];
	}
}

- (void)alphaButtonsFinished:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context
{	
	//NSLog(@"alphaButtonsFinished");
	if( alphaButtonsDisplayed )
	{
		for( int i = 0; i < _HotspotObj.size(); ++i )
			hotspotImgView[i].alpha = 0.25;
		
		[UIView beginAnimations:nil context:self]; 
		{
			[UIView setAnimationDuration:0.35];
			[UIView setAnimationDelegate:self];
			[UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
			[UIView setAnimationDidStopSelector:@selector(alphaButtonsBeginFinished:finished:context:)];
			for( int i = 0; i < _HotspotObj.size(); ++i )
				hotspotImgView[i].alpha = 1;
		}
		[UIView commitAnimations];
	}
}

- (void)alphaButtonsBeginFinished:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context
{	
	//NSLog(@"alphaBeginFinished");
	if( alphaButtonsDisplayed )
		[self beginAlphaButtons];
}

-(void)alphaAnimation
{
	//update hotspots
	gg = (Gob::GobEngine*)g_engine;	
	gg->_goblin->updateHotspot();
	
	//for each one of the scene, we displayed them with fade in/out
	for( int i = 0; i < _HotspotObj.size(); ++i )
	{
		int hotspotCenterX = (_HotspotObj[i]->left + (_HotspotObj[i]->right - _HotspotObj[i]->left)/2)*480/320;
		int hotspotCenterY = (_HotspotObj[i]->top  + (_HotspotObj[i]->bottom - _HotspotObj[i]->top)/2)*320/200;
		
		//alloc image view & image
		UIImageView* iv = [[UIImageView alloc] initWithFrame:CGRectMake(hotspotCenterX - 5, hotspotCenterY - 5, 20, 20)];		
		hotspotImg = [[UIImage imageNamed:@"point d'exclamation bleu.png"] retain];		
		[iv setImage:hotspotImg];		
		[hotspotImg release];		
		[self addSubview:iv];		
		
		hotspotImgView[i] = iv;
	}
	
	//we fade in/out the buttons too
	alphaButtonsDisplayed = true;
	[self beginAlphaButtons];
	
	next = true;
}

-(void)beginAnimationButtons:(CGRect)rect
{
	if( bAnimatingButtons )
	{
		if( pointerButton == nil )
		{
			pointerButton = [[UIImageView alloc] initWithFrame:rect];
			switch( etatTuto )
			{
				case 7:
				case 0: pointerButton.transform = CGAffineTransformMakeRotation(6*3.14/4); break;
				case 1:case 2: pointerButton.transform = CGAffineTransformMakeRotation(0*3.14/4); break;
				case 6: pointerButton.transform = CGAffineTransformMakeRotation(1*3.14/4); break;					
			}
			UIImage* f = [[UIImage imageNamed:@"fleche jaune.png"] retain];		
			[pointerButton setImage:f];		
			[f release];		
			[self addSubview:pointerButton];
		}
		
		switch( etatTuto )
		{
			case 1:case 2: menuButton.alpha = 1; break;
			case 6: hintButton.alpha = 1; break;
		}
		pointerButton.alpha = 1;
		[UIView beginAnimations:nil context:self]; 
		{
			[UIView setAnimationDuration:0.35];
			[UIView setAnimationDelegate:self];
			[UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
			[UIView setAnimationDidStopSelector:@selector(animationButtonsFinished:finished:context:)];
			switch( etatTuto )
			{
				case 1:case 2: menuButton.alpha = 0.25; break;
				case 6: hintButton.alpha = 0.25; break;
			}
			pointerButton.alpha = 0.25;
		}
		[UIView commitAnimations];
	}
}

- (void)animationButtonsFinished:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context
{	
	if( bAnimatingButtons )
	{
		switch( etatTuto )
		{
			case 1:case 2: menuButton.alpha = 0.25; break;
			case 6: hintButton.alpha = 0.25; break;	
		}	
		pointerButton.alpha = 0.25;
		[UIView beginAnimations:nil context:self]; 
		{
			[UIView setAnimationDuration:0.35];
			[UIView setAnimationDelegate:self];
			[UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
			[UIView setAnimationDidStopSelector:@selector(animationButtonsBeginFinished:finished:context:)];
			switch( etatTuto )
			{
				case 1:case 2: menuButton.alpha = 1; break;
				case 6: hintButton.alpha = 1; break;
			}
			pointerButton.alpha = 1;
		}
		[UIView commitAnimations];
	}
}

- (void)animationButtonsBeginFinished:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context
{	
	if( bAnimatingButtons )
		[self beginAnimationButtons:pointerButtonRect];
}

-(void)animNextButton
{
	bAnimatingButtons = true;
	next = true;
	switch( etatTuto )
	{
		case 1:	pointerButtonRect = CGRectMake(442, 260, 20, 20); break;	
		case 2:	pointerButtonRect = CGRectMake(442, 260, 20, 20); break;				
		case 6: pointerButtonRect = CGRectMake(310, 270, 20, 20); break;
		case 7: pointerButtonRect = CGRectMake(255, 125, 20, 20); break;			
	}
	[self beginAnimationButtons:pointerButtonRect];	
}

-(void)sleep
{
	bSleepMode = true;
	next = true;
}

-(IBAction)nextTuto:(id)sender
{
	if( bTutoOn )
	{
		switch( etatTuto )
		{
			case 0: [self tutoStepTwo]; break;
			case 1: [self tutoStepThree]; break;
			case 2: [self tutoStepFour]; break;
			case 3: [self tutoStepFive]; break;	
			case 4: [self tutoStepSix]; break;
			case 5: [self tutoStepSeven]; break;
			case 6: [self tutoStepEight]; break;	
			case 7: [self endTuto]; break;	
		}
	}
}

-(void)tutoStepOne
{
	bProtecOFF = true;	
	etatTuto = 0;
	
	//bar tuto
	tutoBar.hidden = NO;
	[tutoBar setUserInteractionEnabled: NO];
	
	//msg begin lvl
	msgBegin = [[UITextView alloc ] initWithFrame:CGRectMake( 8, 148, 480, 100 )]; //( 8, -7, 480, 100 )
	msgBegin.backgroundColor = [UIColor clearColor];
	msgBegin.textColor = [UIColor whiteColor];
	msgBegin.font = [UIFont fontWithName:@"Verdana" size:13];
	msgBegin.scrollEnabled = NO;
	msgBegin.editable = NO;
	msgBegin.autoresizingMask = UIViewAutoresizingFlexibleHeight;
	msgBegin.text = NSLocalizedString(@"Tuto1Text", @"You can touch the crystal ball in the bottom bar to select a gobliiin");
	[self addSubview:msgBegin];
	[msgBegin setUserInteractionEnabled: NO];	
	
	//header text
	if( _GobLanguage == 0 )
		tutoTitleV = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, 480, 27)];		
	else if( _GobLanguage == 1 )
		tutoTitleV = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, 480, 27)];				
	UIImage* tutoImg = [[UIImage imageNamed:NSLocalizedString(@"TutoTitleImg", "tutorial_title.png")] retain];		
	[tutoTitleV setImage:tutoImg];		
	[tutoImg release];		
	[self addSubview:tutoTitleV];
	
	//tuto button
	tutoIcon = [[UIButton buttonWithType:UIButtonTypeCustom] retain];
	tutoIcon.frame = CGRectMake( 445, 185, 31, 26 );
	[tutoIcon setImage:[UIImage imageNamed:@"Bouton suivant.png"] forState:UIControlStateNormal];
	[tutoIcon addTarget:self action:@selector(nextTuto:) forControlEvents:UIControlEventTouchDown];
	tutoIcon.exclusiveTouch = YES;
	[self addSubview:tutoIcon];
	
	bAnimatingButtons = true;
	pointerButtonRect = CGRectMake(200, 285, 20, 20);
	[self beginAnimationButtons:pointerButtonRect];
	
	if( [[UIDevice currentDevice].systemVersion floatValue] > 3.2)
		[NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(sleep) userInfo:nil repeats:NO];
	else
		[NSTimer scheduledTimerWithTimeInterval:2 target:self selector:@selector(sleep) userInfo:nil repeats:NO];
}

-(void)tutoStepTwo
{
	if( next )
	{
		next = false;
		bAnimatingButtons = false;	
		if( pointerButton != nil && pointerButton.superview != nil )
		{
			[pointerButton removeFromSuperview];
			pointerButton = nil;
		}
		etatTuto = 1;
		
		//msg begin lvl
		msgBegin.frame = CGRectMake( 20, 148, 480, 100 );
		msgBegin.font = [UIFont fontWithName:@"Verdana" size:13];
		msgBegin.text = NSLocalizedString(@"Tuto2Text", @"You can restart the current level and change the settings here");
		
		[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(animNextButton) userInfo:nil repeats:NO];
	}
}

-(void)tutoStepThree
{
	if( next )
	{
		etatTuto = 2;
		menuButton.alpha = 1;
		bAnimatingButtons = false;
		if( pointerButton != nil && pointerButton.superview != nil )
		{
			[pointerButton removeFromSuperview];
			pointerButton = nil;
		}
		
		//msg begin lvl
		msgBegin.frame = CGRectMake( 3, 148, 480, 100 );
		msgBegin.font = [UIFont fontWithName:@"Verdana" size:13];
		msgBegin.text = NSLocalizedString(@"Tuto3Text", @"In the settings menu, you will find 2 control modes : cursor or touch");
		
		[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(animNextButton) userInfo:nil repeats:NO];
	}
}

-(void)tutoStepFour
{
	if( next )
	{
		etatTuto = 3;
		menuButton.alpha = 1;
		bAnimatingButtons = false;
		if( pointerButton != nil && pointerButton.superview != nil )
		{
			[pointerButton removeFromSuperview];
			pointerButton = nil;
		}
		
		//msg begin lvl
		if( _GobLanguage == 0 )
			msgBegin.frame = CGRectMake( 35, 148, 480, 100 );
		else if( _GobLanguage == 1 )
			msgBegin.frame = CGRectMake( 15, 148, 480, 100 );			
		msgBegin.font = [UIFont fontWithName:@"Verdana" size:13];
		msgBegin.text = NSLocalizedString(@"Tuto4Text", @"In cursor mode, drag the cursor and then choose an action");
	}
}

-(void)tutoStepFive
{
	etatTuto = 4;
	
	//msg begin lvl
	if( _GobLanguage == 0 )
		msgBegin.frame = CGRectMake( 50, 148, 480, 100 );
	else if( _GobLanguage == 1 )
		msgBegin.frame = CGRectMake( 7, 148, 480, 100 );		
	msgBegin.font = [UIFont fontWithName:@"Verdana" size:13];
	msgBegin.text = NSLocalizedString(@"Tuto5Text", @"In touch mode, just tap the screen to move a gobliiin");
}

-(void)tutoStepSix
{
	if( next )
	{
		next = false;
		etatTuto = 5;
		
		//msg begin lvl
		if( _GobLanguage == 0 )
			msgBegin.frame = CGRectMake( 30, 148, 480, 100 );
		else if( _GobLanguage == 1 )
			msgBegin.frame = CGRectMake( 6, 148, 480, 100 );			
		msgBegin.font = [UIFont fontWithName:@"Verdana" size:13];
		msgBegin.text = NSLocalizedString(@"Tuto6Text", @"Just move your finger/cursor around to display the hotspots");
		
		[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(alphaAnimation) userInfo:nil repeats:NO];
	}
}

-(void)tutoStepSeven
{
	if( next )
	{
		next = false;
		alphaButtonsDisplayed = false;
		for( int i = 0; i < _HotspotObj.size(); ++i )
		{
			if( hotspotImgView[i] != nil && hotspotImgView[i].superview != nil )
			{
				[hotspotImgView[i] removeFromSuperview];
				hotspotImgView[i] = nil;
			}
		}
		etatTuto = 6;
		
		//msg begin lvl
		if( _GobLanguage == 0 )
			msgBegin.frame = CGRectMake( 76, 148, 480, 100 );
		else if( _GobLanguage == 1 )
			msgBegin.frame = CGRectMake( 68, 148, 480, 100 );			
		msgBegin.font = [UIFont fontWithName:@"Verdana" size:13];
		msgBegin.text = NSLocalizedString(@"Tuto7Text", @"If you need some help, just look at the hints!");
		
		[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(animNextButton) userInfo:nil repeats:NO];
	}
}

-(void)tutoStepEight
{
	if( next )
	{
		next = false;
		bAnimatingButtons = false;
		hintButton.alpha = 1.0;	
		if( pointerButton != nil && pointerButton.superview != nil )
		{
			[pointerButton removeFromSuperview];
			pointerButton = nil;
		}
		etatTuto = 7;
		
		//msg begin lvl
		if( _GobLanguage == 0 )
			msgBegin.frame = CGRectMake( 15, 148, 480, 100 );
		else if( _GobLanguage == 1 )
			msgBegin.frame = CGRectMake( 3, 148, 480, 100 );			
		msgBegin.font = [UIFont fontWithName:@"Verdana" size:13];
		msgBegin.text = NSLocalizedString(@"Tuto8Text", @"Now, try to punch the pillar. And remember to look at the hints!");
		
		[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(alphaAnimation) userInfo:nil repeats:NO];
		[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(animNextButton) userInfo:nil repeats:NO];
	}
}

-(void)protection
{
	bProtecOFF = false;
}

-(void)endTuto
{
	if( next ) 
	{
		next = false;
		bAnimatingButtons = false;	
		bTutoOn = false;
		[NSTimer scheduledTimerWithTimeInterval:0.6 target:self selector:@selector(protection) userInfo:nil repeats:NO];		
		alphaButtonsDisplayed = false;
		bSleepMode = false;
		if( bSem )
		{
			sem_post( sem );		
			bSem = false;
		}
		
		for( int i = 0; i < _HotspotObj.size(); ++i )
		{
			if( hotspotImgView[i] != nil && hotspotImgView[i].superview != nil )
			{
				[hotspotImgView[i] removeFromSuperview];
				hotspotImgView[i] = nil;
			}
		}
		if( pointerButton != nil && pointerButton.superview != nil )
		{
			[pointerButton removeFromSuperview];
			pointerButton = nil;
		}
		if( tutoBar != nil && tutoBar.superview != nil )
		{
			tutoBar.hidden = YES;
		}
		if( msgBegin != nil && msgBegin.superview != nil )
		{
			[msgBegin removeFromSuperview];
			msgBegin = nil;
		}
		if( tutoIcon != nil && tutoIcon.superview != nil )
		{
			[tutoIcon removeFromSuperview];
			tutoIcon = nil;
		}
		if( tutoTitleV != nil && tutoTitleV.superview != nil )
		{
			[tutoTitleV removeFromSuperview];
			tutoTitleV = nil;
		}
		
		//cursor mouse
		if( bMouseModeOn )
		{
			reset = false;
			[self cursorCurrentGobliiin:((OSystem_IPHONE*)g_system)->_mouseX coordY:((OSystem_IPHONE*)g_system)->_mouseY];
		}
	}
}

-(void)endTutoAgain
{
	//NSLog(@"endTutoAgain");
	next = true;
	[self endTuto];
}

-(void)touchMode
{
	if( cursorGobliiin != nil && cursorGobliiin.superview != nil )
	{
		[cursorGobliiin removeFromSuperview];
		cursorGobliiin = nil;
		cursorGobliiin.hidden = YES;
	}
}

-(void)cursorMode
{
	[self cursorCurrentGobliiin	:((OSystem_IPHONE*)g_system)->_mouseX coordY:((OSystem_IPHONE*)g_system)->_mouseY];
}

-(void) cursorCurrentGobliiin:(int)xxx coordY:(int)yyy
{
	//NSLog(@"iPhoneViewController cursorCurrentGobliiin");	
	if( cursorGobliiin != nil && cursorGobliiin.superview != nil )
	{
		[cursorGobliiin removeFromSuperview];
		cursorGobliiin = nil;
	}

	cursorGobliiin = [[UIImageView alloc] initWithFrame:CGRectMake(xxx*480/320, yyy*320/200, 40, 40)];	
	UIImage* curs;
	
	switch( numGobliiin )
	{
		case 0:
			curs = [[UIImage imageNamed:@"curgob1.png"] retain];		
			[cursorGobliiin setImage:curs];		
			[curs release];	
			break;
		case 1:
			curs = [[UIImage imageNamed:@"curgob2.png"] retain];		
			[cursorGobliiin setImage:curs];		
			[curs release];	
			break;
		case 2:
			curs = [[UIImage imageNamed:@"curgob3.png"] retain];		
			[cursorGobliiin setImage:curs];		
			[curs release];	
			break;
		default:
			curs = [[UIImage imageNamed:@"curgob1.png"] retain];		
			[cursorGobliiin setImage:curs];		
			[curs release];	
			break;
	}
	cursorGobliiin.hidden = NO;
	[self addSubview:cursorGobliiin];
}

-(void) majGobliiin
{
	//NSLog(@"iPhoneViewController majGobliiin");	
	UIImage* curs;
	switch( numGobliiin )
	{
		case 0:
			curs = [[UIImage imageNamed:@"curgob1.png"] retain];		
			cursorGobliiin.image = nil;
			cursorGobliiin.image = curs;
			[curs release];	
			break;
		case 1:
			curs = [[UIImage imageNamed:@"curgob2.png"] retain];		
			cursorGobliiin.image = nil;
			cursorGobliiin.image = curs;		
			[curs release];	
			break;
		case 2:
			curs = [[UIImage imageNamed:@"curgob3.png"] retain];		
			cursorGobliiin.image = nil;
			cursorGobliiin.image = curs;		
			[curs release];	
			break;
		default:
			curs = [[UIImage imageNamed:@"curgob1.png"] retain];		
			cursorGobliiin.image = nil;
			cursorGobliiin.image = curs;	
			[curs release];	
			break;
	}
}

-(void)resumeLevel
{
	//loading fade out
	[[NSNotificationCenter defaultCenter] postNotificationName:@"loadingScreenOff" object:nil];	
	
	//etat action
	if( mEtatAction != 0 )
		[self resetAction:0 coordY:0];
	
	//buttons appear
	choiceButton.hidden = NO;
	hintButton.hidden = NO;
	menuButton.hidden = NO;	
	barreInventaire.hidden = NO;
	
	//clear
	bFromMenuIngame = false;
	
	//animation alpha lvl 1
	if( idLvLCourant == 1 )
	{
		bTutoOn = true;
		[self tutoStepOne];
	}
	else
	{
		[self endTuto];
	}
	
	if( idLvLCourant == 3 ) 
	{
		//orientation
		if( [[UIDevice currentDevice] orientation] == UIDeviceOrientationLandscapeLeft && [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationLandscapeLeft )
			[[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeLeft animated:NO];
		else if( [[UIDevice currentDevice] orientation] == UIDeviceOrientationLandscapeRight && [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationLandscapeRight )
			[[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeRight animated:NO];
		/*else
		{
			if( [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationPortrait || [[UIDevice currentDevice] orientation] == UIDeviceOrientationPortrait )
				[[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeRight animated:NO];
		}*/
		
		[[BulkyServicesView sharedInstance] promptRating:YES];
	}
}

-(void)levelLoaded
{
	NSLog(@"iPhoneView levelLoaded");	
	if( [[UIDevice currentDevice].systemVersion floatValue] > 3.2)
		[NSTimer scheduledTimerWithTimeInterval:0 target:self selector:@selector(resumeLevel) userInfo:nil repeats:NO];
	else 
		[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(resumeLevel) userInfo:nil repeats:NO];	
}

-(void)hide
{
	//buttons
	choiceButton.hidden = YES;
	hintButton.hidden = YES;
	menuButton.hidden = YES;
	barreInventaire.hidden = YES;
	cursorGobliiin.hidden = YES;
}

-(void)hideButtons
{
	//NSLog(@"iPhoneView hideButtons");
	[NSTimer scheduledTimerWithTimeInterval:0.25 target:self selector:@selector(hide) userInfo:nil repeats:NO];
}

-(void)skipButtonOn
{
	//NSLog(@"iPhoneView skipButtonOn");
	skipButton.hidden = NO;
}

-(void)skipButtonOff
{
	//NSLog(@"iPhoneView skipButtonOff");
	skipButton.hidden = YES;
}

-(void)endAnimEndGame
{
	//NSLog(@"iPhoneView endAnimEndGame");	
	bMenuIngame = false;
	bFromMenuIngame = true;
	bMenuGob1 = true;
	[g_switchVC switchViews];
	[audioPlayer stop];	
	idLvLCourant = -1;
}

-(void)endGame
{
	//NSLog(@"iPhoneView endGame");
	
	//popup
	bPopup = true;
	
	//buttons
	[self hideButtons];
	
	//clear everything
	[self clearButtons];	
	
	if( bMouseModeOn )
	{
		if( cursorGobliiin != nil && cursorGobliiin.superview != nil )
		{
			[cursorGobliiin removeFromSuperview];
			cursorGobliiin = nil;
		}
	}
	
	skipButton.hidden = NO;
	
	//orientation
	if( [[UIDevice currentDevice] orientation] == UIDeviceOrientationLandscapeLeft && [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationLandscapeLeft )
		[[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeLeft animated:NO];
	else if( [[UIDevice currentDevice] orientation] == UIDeviceOrientationLandscapeRight && [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationLandscapeRight )
		[[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeRight animated:NO];
	/*else
	 {
	 if( [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationPortrait || [[UIDevice currentDevice] orientation] == UIDeviceOrientationPortrait )
	 [[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeRight animated:NO];
	 }*/
	
	//popup
	UIImage *backgroundImage = [UIImage imageNamed:@"popup_fond.png"];
	UIImage *img1button = [[UIImage imageNamed:@"popup_ok_big.png"] retain];
	alert = nil;
	alert = [[DECustomAlert alloc] initWithImage:backgroundImage text:NSLocalizedString(@"EndGameTitle", @"Game Over!") sstext:NSLocalizedString(@"EndGameText", @"Congratulations! You defeat the horrible wizard. Enjoy the parade!") button1:true img1:img1button button2:false img2:nil];
	alert.delegate = self;
	[alert show];
	[backgroundImage release];
	[img1button release];
	iPhoneViewCall = true;
	
	NSURL *url = [NSURL fileURLWithPath:[NSString stringWithFormat:@"%@/blop.mp3", [[NSBundle mainBundle] resourcePath]]];
	NSError *error;
	popupSound = [[AVAudioPlayer alloc] initWithContentsOfURL:url error:&error];
	popupSound.volume = (float)((float)idSoundSFX / 256.0f);
	popupSound.numberOfLoops = 0;
	[popupSound play];
}
//END NOTIF

-(void) dealloc
{
	//NSLog(@"iPhoneView dealloc");	
	[super dealloc];

	[skipButton release];
	[choiceButton release];
	[hintButton release];
	[menuButton release];
	[barreInventaire release];	
	
	[useIcon release];
	[pickIcon release];
	[stepIcon release];
	[cancelIcon release];	
	
	[audioPlayer release];
	[loadingV release];
	
	delete gg;

	_screenLayer = nil;
	sharedInstance = nil;
	_context = nil;
	_screenTexture = 0;

	if (_screenTexture)
		free(_textureBuffer);
}

- (void *)getSurface 
{
	return _screenSurface;
}

- (void)updateSurface 
{
	if( !bBackground )
	{	
		if (!_needsScreenUpdate) 
		{
			return;
		}
		_needsScreenUpdate = 0;
		
		GLfloat vertices[] =
		{
			0.0f + _heightOffset, 0.0f + _widthOffset,
			_visibleWidth - _heightOffset, 0.0f + _widthOffset,
			0.0f + _heightOffset,  _visibleHeight - _widthOffset,
			_visibleWidth - _heightOffset,  _visibleHeight - _widthOffset
		};
		
		float texWidth = _width / (float)_textureWidth;
		float texHeight = _height / (float)_textureHeight;
		
		const GLfloat texCoords[] =
		{
			texWidth, 0.0f,
			0.0f, 0.0f,
			texWidth, texHeight,
			0.0f, texHeight
		};
		
		glVertexPointer(2, GL_FLOAT, 0, vertices);
		glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
		
		if( !bSimulation ) //si on simule pas un niveau, on dessine, sinon on dessine rien
		{
			//[_lock lock];
			// Unfortunately we have to update the whole texture every frame, since glTexSubImage2D is actually slower in all cases
			// due to the iPhone internals having to convert the whole texture back from its internal format when used.
			// In the future we could use several tiled textures instead.
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, _textureWidth, _textureHeight, 0, GL_RGB, GL_UNSIGNED_SHORT_5_6_5, _textureBuffer);
			//[_lock unlock];
			
			glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
			glBindRenderbufferOES(GL_RENDERBUFFER_OES, _viewRenderbuffer);
			[_context presentRenderbuffer:GL_RENDERBUFFER_OES];
		}
	}
}

- (void)initSurface
{
	//NSLog(@"iPhoneView initSurface");		
	_textureWidth = getSizeNextPOT(_width);
	_textureHeight = getSizeNextPOT(_height);
	
	_screenOrientation = [UIDevice currentDevice].orientation;//UIDeviceOrientationLandscapeRight;
	
	//printf("Window: (%d, %d), Surface: (%d, %d), Texture(%d, %d)\n", _fullWidth, _fullHeight, _width, _height, _textureWidth, _textureHeight);
	
	if (_context == nil)
	{
		_screenOrientation = [UIDevice currentDevice].orientation;//UIDeviceOrientationLandscapeRight;
		CAEAGLLayer *eaglLayer = (CAEAGLLayer*) self.layer;
		
		eaglLayer.opaque = YES;
		eaglLayer.drawableProperties = [NSDictionary dictionaryWithObjectsAndKeys:
										[NSNumber numberWithBool:FALSE], kEAGLDrawablePropertyRetainedBacking, kEAGLColorFormatRGB565, kEAGLDrawablePropertyColorFormat, nil];
		
		_context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
		if (!_context || [EAGLContext setCurrentContext:_context])
		{
			glGenFramebuffersOES(1, &_viewFramebuffer);
			glGenRenderbuffersOES(1, &_viewRenderbuffer);
			
			glBindFramebufferOES(GL_FRAMEBUFFER_OES, _viewFramebuffer);
			glBindRenderbufferOES(GL_RENDERBUFFER_OES, _viewRenderbuffer);
			[_context renderbufferStorage:GL_RENDERBUFFER_OES fromDrawable:(id<EAGLDrawable>)self.layer];
			glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_COLOR_ATTACHMENT0_OES, GL_RENDERBUFFER_OES, _viewRenderbuffer);
			
			glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_WIDTH_OES, &_backingWidth);
			glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_HEIGHT_OES, &_backingHeight);
			
			//printf(" backing %d; %d \n", _backingWidth, _backingHeight );
			
			if (glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES) != GL_FRAMEBUFFER_COMPLETE_OES) 
			{
				//NSLog(@"Failed to make complete framebuffer object %x.", glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES));
				return;
			}
			
			_backingWidth = 480;
			_backingHeight = 600;
			glViewport(0, -140, _backingWidth, _backingHeight);
			//printf("glViewport - 0 0 %d %d\n", _backingWidth, _backingHeight);
			glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
			
			glEnable(GL_TEXTURE_2D);
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
			glEnableClientState(GL_VERTEX_ARRAY);
		}
	}
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	if (_screenOrientation ==  UIDeviceOrientationLandscapeRight)
	{
		glRotatef(-180, 0, 0, 1); //-90
	} 
	else if (_screenOrientation == UIDeviceOrientationLandscapeLeft) 
	{
		glRotatef(180, 0, 0, 1); //90		
	} 
	else 
	{
		_screenOrientation = UIDeviceOrientationLandscapeRight;
		glRotatef(-180, 0, 0, 1); //-180
	}
	
	glOrthof(0, _backingWidth, 0, _backingHeight, 0, 1);
	//printf("glOrthof - 0 %d 0 %d 0 1\n", _backingWidth, _backingHeight);
	
	if (_screenTexture > 0)
	{
		glDeleteTextures(1, &_screenTexture);
	}
	
	glGenTextures(1, &_screenTexture);
	glBindTexture(GL_TEXTURE_2D, _screenTexture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	
	if (_textureBuffer) 
	{
		free(_textureBuffer);
	}
	
	int textureSize = _textureWidth * _textureHeight * 2;
	_textureBuffer = (char*)malloc(textureSize);
	memset(_textureBuffer, 0, textureSize);
	
	glBindRenderbufferOES(GL_RENDERBUFFER_OES, _viewRenderbuffer);
	
	// The color buffer is triple-buffered, so we clear it multiple times right away to avid doing any glClears later.
	/*int clearCount = 5;
	while (clearCount-- > 0)
	{*/
		glClear(GL_COLOR_BUFFER_BIT);
		[_context presentRenderbuffer:GL_RENDERBUFFER_OES];
	//}
	
	if (_screenOrientation == UIDeviceOrientationLandscapeLeft || _screenOrientation ==  UIDeviceOrientationLandscapeRight)
	{
		_visibleHeight = _backingHeight;
		_visibleWidth = _backingWidth;
		
		float ratioDifference = ((float)_height / (float)_width) / ((float)_fullWidth / (float)_fullHeight);
		int rectWidth, rectHeight;
		
		if (ratioDifference < 1.0f) 
		{
			rectWidth = _fullWidth * ratioDifference;
			rectHeight = _fullHeight;
			_widthOffset = (_fullWidth - rectWidth) / 2;
			_heightOffset = 0;
		} 
		else 
		{
			rectWidth = _fullWidth;
			rectHeight = _fullHeight / ratioDifference;
			_heightOffset = (_fullHeight - rectHeight) / 2;
			_widthOffset = 0;
		}
		
		//printf("Rect: %i, %i, %i, %i\n", _widthOffset, _heightOffset, rectWidth, rectHeight);
		_screenRect = CGRectMake(_widthOffset, _heightOffset, rectWidth, rectHeight);
	} 
	else
	{
		float ratio = (float)_height / (float)_width;
		int height = _fullWidth * ratio;
		//printf("Making rect (%u, %u)\n", _fullWidth, height);
		_screenRect = CGRectMake(0, 0, _fullWidth - 1, height - 1);
		
		_visibleHeight = height;
		_visibleWidth = _backingWidth;
		_heightOffset = 0.0f;
		_widthOffset = 0.0f;
		
		CGRect keyFrame = CGRectMake(0.0f, 0.0f, 0.0f, 0.0f);
	}
}

- (id)getEvent
{	
	if (_events == nil || [_events count] == 0) 
		return nil;
	
	id event = [_events objectAtIndex: 0];
	
	[_events removeObjectAtIndex: 0];
	
	return event;
}

- (void)addEvent:(NSDictionary*)event 
{	
	if (_events == nil)
		_events = [[NSMutableArray alloc] init];
	
	[_events addObject: event];
}

- (void)deviceOrientationChanged:(UIDeviceOrientation)orientation 
{		
	if( orientation == UIDeviceOrientationLandscapeLeft )
	{
		_screenOrientation = orientation;
		//[[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeLeft animated:NO];
	}
	else if( orientation == UIDeviceOrientationLandscapeRight )
	{
		_screenOrientation = orientation;
		//[[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeRight animated:NO];	
	}
	else 
	{
		if( orientation == UIDeviceOrientationPortrait )
		{
			_screenOrientation = UIDeviceOrientationLandscapeRight;
			//[[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeRight animated:NO];
		}
	}
}

- (void)clearIcons
{	
	if( useIcon != nil && useIcon.superview != nil )
		[useIcon removeFromSuperview];
	useIcon.hidden = YES;
	if( pickIcon != nil && pickIcon.superview != nil )
		[pickIcon removeFromSuperview];
	pickIcon.hidden = YES;
	if( stepIcon != nil && stepIcon.superview != nil )
		[stepIcon removeFromSuperview];
	stepIcon.hidden = YES;
	if( cancelIcon != nil && cancelIcon.superview != nil )
		[cancelIcon removeFromSuperview];
	cancelIcon.hidden = YES;		
}

- (void)iconsPositionCompute:(float)xPos coordY:(float)yPos
{	
	xPos = xPos*480/320;
	yPos = yPos*320/200;
	
	if( xPos <= 40 ) //gauche
	{
		stepIcon.frame = CGRectMake( xPos + 10, yPos - 36 - 10, 36, 36 );
		useIcon.frame = CGRectMake( xPos + 36 + 20, yPos - 36 - 10, 36, 36 );	
		cancelIcon.frame = CGRectMake( xPos + 10, yPos + 20, 36, 36 );		
		pickIcon.frame = CGRectMake( xPos + 36 + 20, yPos + 20, 36, 36 );	
	}
	else if( xPos >= 420 ) //droite
	{
		stepIcon.frame = CGRectMake( xPos - 36 - 10, yPos - 36 - 20, 36, 36 );		
		useIcon.frame = CGRectMake( xPos - 36 - 36 - 20, yPos - 36 - 20, 36, 36 );	
		cancelIcon.frame = CGRectMake( xPos - 36 - 10, yPos + 20, 36, 36 );		
		pickIcon.frame = CGRectMake( xPos - 36 - 36 - 20, yPos + 20, 36, 36 );		
	}
	else if( yPos <= 48 ) //haut
	{
		stepIcon.frame = CGRectMake( xPos - 36 - 10, yPos + 36 + 30, 36, 36 );
		useIcon.frame = CGRectMake( xPos - 36 - 10, yPos + 20, 36, 36 );			
		cancelIcon.frame = CGRectMake( xPos + 20, yPos + 36 + 30, 36, 36 );
		pickIcon.frame = CGRectMake( xPos + 20, yPos + 20, 36, 36 );		
	}
	else if( yPos >= 210 ) //bas
	{
		stepIcon.frame = CGRectMake( xPos - 36 - 10, yPos - 36 - 30, 36, 36 );
		useIcon.frame = CGRectMake( xPos - 36 - 10, yPos - 20, 36, 36 );			
		cancelIcon.frame = CGRectMake( xPos + 20, yPos - 36 - 30, 36, 36 );
		pickIcon.frame = CGRectMake( xPos + 20, yPos - 20, 36, 36 );			
	}
	else
	{
		stepIcon.frame = CGRectMake( xPos - 13, yPos - 36 - 15, 36, 36 );
		cancelIcon.frame = CGRectMake( xPos - 13, yPos + 25, 36, 36 );
		useIcon.frame = CGRectMake( xPos - 36 - 10, yPos - 16, 36, 36 );	
		pickIcon.frame = CGRectMake( xPos + 20, yPos - 16, 36, 36 );	
	}	
	//corners
	if( xPos <= 40 && yPos <= 48 ) //corner top left
	{
		stepIcon.frame = CGRectMake( xPos + 15, yPos, 36, 36 );
		useIcon.frame = CGRectMake( xPos + 36 + 25, yPos, 36, 36 );	
		cancelIcon.frame = CGRectMake( xPos + 15, yPos + 20 + 36, 36, 36 );
		pickIcon.frame = CGRectMake( xPos + 36 + 25, yPos + 20 + 36, 36, 36 );	
	}
	else if( xPos <= 40 && yPos >= 210 ) //corner bottom left
	{
		stepIcon.frame = CGRectMake( xPos + 15, yPos - 36 - 36 - 30, 36, 36 );
		useIcon.frame = CGRectMake( xPos + 36 + 25, yPos - 36 - 36 - 30, 36, 36 );	
		cancelIcon.frame = CGRectMake( xPos + 15, yPos - 36 - 10, 36, 36 );
		pickIcon.frame = CGRectMake( xPos + 36 + 25, yPos - 36 - 10, 36, 36 );	
	}
	else if( xPos >= 420 &&  yPos <= 48 ) //corner top right
	{
		stepIcon.frame = CGRectMake( xPos - 36 - 10, yPos, 36, 36 );
		useIcon.frame = CGRectMake( xPos - 36 - 36 - 20, yPos, 36, 36 );	
		cancelIcon.frame = CGRectMake( xPos - 36 - 36 - 20, yPos + 36 + 10, 36, 36 );
		pickIcon.frame = CGRectMake( xPos - 36 - 10, yPos + 36 + 10, 36, 36 );
	}
	else if( xPos >= 420 && yPos >= 210 ) //corner bottom right
	{
		stepIcon.frame = CGRectMake( xPos - 36 - 10, yPos - 36 - 36 - 30, 36, 36 );
		useIcon.frame = CGRectMake( xPos - 36 - 36 - 20, yPos - 36 - 36 - 30, 36, 36 );	
		cancelIcon.frame = CGRectMake( xPos - 36 - 36 - 20, yPos - 36 - 10, 36, 36 );
		pickIcon.frame = CGRectMake( xPos - 36 - 10, yPos - 36 - 10, 36, 36 );
	}			
	//add to view
	useIcon.hidden = NO;
	switch( numGobliiin )
	{
		case 0:	[useIcon setImage:[UIImage imageNamed:@"engrenage.png"] forState:UIControlStateNormal];
			break;
		case 1:	[useIcon setImage:[UIImage imageNamed:@"Marron-poing3.png"] forState:UIControlStateNormal];
			break;
		case 2:	[useIcon setImage:[UIImage imageNamed:@"baguette.png"] forState:UIControlStateNormal];
			break;
	}
	[self addSubview:useIcon];			
	if( bGobPicker )
	{
		pickIcon.hidden = NO;
		[self addSubview:pickIcon];
	}
	stepIcon.hidden = NO;
	[self addSubview:stepIcon];			
	cancelIcon.hidden = NO;
	[self addSubview:cancelIcon];
}

- (int)hotspotDistance:(CGPoint)point
{	
	float hotspotCenterX;
	float hotspotCenterY;
	int distX[20] = { 0 };
	int distY[20] = { 0 };
	
	//get the dist between the touch point and the center of the hotspot for each one
	for( int i = 0; i < _HotspotObj.size(); ++i )
	{
		hotspotCenterX = (_HotspotObj[i]->left + (_HotspotObj[i]->right - _HotspotObj[i]->left)/2)*480/320;
		hotspotCenterY = (_HotspotObj[i]->top  + (_HotspotObj[i]->bottom - _HotspotObj[i]->top)/2)*320/200;		
		distX[i] = abs( point.x - hotspotCenterX*320/480 );
		distY[i] = abs( point.y - hotspotCenterY*200/320 );
	}	
	
	int minx = 10, miny = 10, ind = -1;
	//we compare then with a minima and we return the one we are in
	for( int i = 0; i < _HotspotObj.size(); ++i )
	{
		if( distX[i] < minx && distY[i] < miny )
		{
			minx = distX[i];
			miny = distY[i];
			ind = i;
		}
	}
	return ind;
}

- (void)hotspotDisplayCompute:(CGPoint)point
{	
	float hotspotCenterX;
	float hotspotCenterY;
	
	int ind = [self hotspotDistance:point];
	
	//PARCOURS DES HOTSPOTS
	for( int i = 0; i < _HotspotObj.size(); ++i )
	{
		hotspotCenterX = (_HotspotObj[i]->left + (_HotspotObj[i]->right - _HotspotObj[i]->left)/2)*480/320;
		hotspotCenterY = (_HotspotObj[i]->top  + (_HotspotObj[i]->bottom - _HotspotObj[i]->top)/2)*320/200;
		
		//GESTION AFFICHAGE DES "!"
		//look if the touch position is in the hotspot
		if( ind != -1 )
		{			
			//clear all the other hotspots
			for( int j = 0; j < _HotspotObj.size(); ++j )
			{
				if( j != ind )
				{
					alphaDisplayed[j] = false;
					hotspotDisplayed[j] = false;
					hotspotImgView[j].hidden = YES;
					hotspotUsed[j] = false;
					[self clearIcons];	
				}
			}
			
			hotspotCenterX = (_HotspotObj[ind]->left + (_HotspotObj[ind]->right - _HotspotObj[ind]->left)/2)*480/320;
			hotspotCenterY = (_HotspotObj[ind]->top  + (_HotspotObj[ind]->bottom - _HotspotObj[ind]->top)/2)*320/200;
			
			if( !hotspotDisplayed[ind] ) //if ! no displayed for this hotspot
			{				
				if( hotspotImgView[ind] == nil )
				{
					//alloc image view & image
					UIImageView* iv = [[UIImageView alloc] initWithFrame:CGRectMake(hotspotCenterX - 5, hotspotCenterY - 5, 20, 20)];		
					hotspotImg = [[UIImage imageNamed:@"point d'exclamation bleu.png"] retain];		
					[iv setImage:hotspotImg];		
					[hotspotImg release];		
					[self addSubview:iv];		
				
					hotspotImgView[ind] = iv;
					[iv release];
				}
				else
				{
					hotspotImgView[ind].hidden = NO;
					hotspotImgView[ind].frame = CGRectMake(hotspotCenterX - 5, hotspotCenterY - 5, 20, 20);
				}
				
				hotspotDisplayed[ind] = true;
			}
			
			//compute the icons position and display them
			[self iconsPositionCompute:hotspotCenterX*320/480 coordY:hotspotCenterY*200/320];
			
			// hide cursor mouse
			bMouseCursorActive = false;
			
			if( cursorGobliiin != nil && cursorGobliiin.superview != nil )
			{
				cursorGobliiin.hidden = YES;
			}

			//the icons are now displayed so don't bother do it again / hotspot used
			hotspotUsed[ind] = true;
			alphaDisplayed[ind] = false;
			hotspotImgView[ind].alpha = 1;
			hotspotImgView[ind].hidden = NO;
		}
		else //vision radius
		if( point.x > _HotspotObj[i]->left - 35 && point.x <= _HotspotObj[i]->right + 35 && point.y > _HotspotObj[i]->top - 35 && point.y <= _HotspotObj[i]->bottom + 35 )
		{			
			hotspotCenterX = (_HotspotObj[i]->left + (_HotspotObj[i]->right - _HotspotObj[i]->left)/2)*480/320;
			hotspotCenterY = (_HotspotObj[i]->top  + (_HotspotObj[i]->bottom - _HotspotObj[i]->top)/2)*320/200;
			
			if( !hotspotDisplayed[i] ) //if ! no displayed for this hotspot
			{				
				if( hotspotImgView[i] == nil )
				{
					//alloc image view & image
					UIImageView* iv = [[UIImageView alloc] initWithFrame:CGRectMake(hotspotCenterX - 5, hotspotCenterY - 5, 20, 20)];		
					hotspotImg = [[UIImage imageNamed:@"point d'exclamation bleu.png"] retain];		
					[iv setImage:hotspotImg];		
					[hotspotImg release];		
					[self addSubview:iv];		
				
					hotspotImgView[i] = iv;
					[iv release];
				}
				else
				{
					hotspotImgView[i].hidden = NO;
					hotspotImgView[i].frame = CGRectMake(hotspotCenterX - 5, hotspotCenterY - 5, 20, 20);
				}
				   
				hotspotDisplayed[i] = true;
				if( !alphaDisplayed[i] )
				{
					alphaDisplayed[i] = true;
					//[self beginAlpha:i];
				}
			}
			hotspotUsed[i] = false;
			hotspotUsed[ind] = false;
			[self clearIcons];	
		}
		else //clear mode
		{			
			alphaDisplayed[i] = false;
			hotspotImgView[i].hidden = YES;			
			hotspotDisplayed[i] = false;
			hotspotUsed[i] = false;

			if( ind != -1 )
			{
				alphaDisplayed[ind] = false;		
				hotspotImgView[ind].hidden = YES;			
				hotspotDisplayed[ind] = false;
				hotspotUsed[ind] = false;					
			}

			[self clearIcons];			
		}
	}	
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{	
	if( !bMenuGob1 && !bGameOverOn && !bPopup && bLvLLoaded && !bMenuIngame && !bHintMenuActive && !bSimulation && !bAnimBeginning && !bLoadingImage )
	{		
		if( !bTutoOn )
		{
			debD = [[NSDate alloc] initWithTimeIntervalSinceNow: 1];
			
			//PAS FIN JEU & IN SCENE : DONC UPDATE
			if( !bPopup && bLvLLoaded && !bMenuIngame && !bHintMenuActive && !bSimulation && !bAnimBeginning )
			{
				//update hotspot
				gg = (Gob::GobEngine*)g_engine;	
				gg->_goblin->updateHotspot();
			}
			
			UITouch *touch = [touches anyObject];
			CGPoint point = [touch locationInView:self];
			
			point.x = point.x * 320 / 480;
			point.y = point.y * 200 / 320;
			if( bMouseModeOn )
			{
				bMouseCursorActive = true;
				dist.x = ((OSystem_IPHONE*)g_system)->_mouseX - point.x;
				dist.y = ((OSystem_IPHONE*)g_system)->_mouseY - point.y;
				point.x = ((OSystem_IPHONE*)g_system)->_mouseX;
				point.y = ((OSystem_IPHONE*)g_system)->_mouseY;
				
				if( cursorGobliiin == nil )
					[self cursorCurrentGobliiin:((OSystem_IPHONE*)g_system)->_mouseX coordY:((OSystem_IPHONE*)g_system)->_mouseY];
				else
					cursorGobliiin.frame = CGRectMake(((OSystem_IPHONE*)g_system)->_mouseX *480/320,((OSystem_IPHONE*)g_system)->_mouseY *320/200, 40, 40);
				cursorGobliiin.hidden = NO;
			}
			g_system->warpMouse(point.x, point.y);
			
			rondV.hidden = YES;
			
			if( !bPopup && !bLoadingLevel && !bSimulation  && !bMenuIngame && !bHintMenuActive )
			{
				//compute the display of the hotspots and buttons	
				[self hotspotDisplayCompute:point];
			}
		}
	}
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{	
	if( !bMenuGob1 && !bGameOverOn && !bPopup && bLvLLoaded && !bMenuIngame && !bHintMenuActive && !bSimulation && !bAnimBeginning && !bLoadingImage )
	{
		if( !bTutoOn )
		{
			//PAS FIN JEU & IN SCENE : DONC UPDATE
			if( !bPopup && bLvLLoaded && !bMenuIngame && !bHintMenuActive && !bSimulation && !bAnimBeginning )
			{
				//update hotspot
				gg = (Gob::GobEngine*)g_engine;	
				gg->_goblin->updateHotspot();
			}
			
			UITouch *touch = [touches anyObject];
			CGPoint point = [touch locationInView:self];
			
			point.x = point.x * 320 / 480;
			point.y = point.y * 200 / 320;
			
			if( bMouseModeOn )
			{
				m.x = point.x + dist.x;
				m.y = point.y + dist.y;
				
				if( m.x < 0 )
					m.x = 0;
				else if( m.x > 313 )
					m.x = 313;
				
				if( m.y < 0 )
					m.y = 0;
				else if( m.y > 160 )
					m.y = 160;
				
				g_system->warpMouse(m.x, m.y);
				
				if( cursorGobliiin != nil )
					cursorGobliiin.frame = CGRectMake(m.x*480/320, m.y*320/200, 40, 40);
				else
					[self cursorCurrentGobliiin:m.x coordY:m.y];
				
				cursorGobliiin.hidden = NO;
			}
			else
				g_system->warpMouse(point.x, point.y);
			
			rondV.hidden = YES;
			
			if( !bPopup && !bLoadingLevel && !bSimulation  && !bMenuIngame && !bHintMenuActive )
			{
				//compute the display of the hotspots and buttons	
				if( bMouseModeOn )
					[self hotspotDisplayCompute:m];
				else
					[self hotspotDisplayCompute:point];
			}
		}
	}
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{			
	if( !bMenuGob1 && !bGameOverOn && !bPopup && bLvLLoaded && !bMenuIngame && !bHintMenuActive && !bSimulation && !bAnimBeginning )
	{
		if( !bTutoOn )
		{
			UITouch *touch = [touches anyObject];
			CGPoint point = [touch locationInView:self];
			
			if( !bMouseModeOn )
			{
				point.x = point.x * 320 / 480;
				point.y = point.y * 200 / 320;
			}
			else
			{
				point.x = ((OSystem_IPHONE*)g_system)->_mouseX;
				point.y = ((OSystem_IPHONE*)g_system)->_mouseY;
			}
			g_system->warpMouse(point.x, point.y);
			
			rondV.hidden = YES;
			
			//if icons displayed do not do events, else clear
			for( int i = 0; i < _HotspotObj.size(); ++i )
			{
				if( hotspotUsed[i] )
				{
					iconsDisplayed = true;
					alphaDisplayed[i] = false;
					hotspotImgView[i].alpha = 1;
					hotspotImgView[i].hidden = NO;
				}
				else
				{
					alphaDisplayed[i] = false;		
					hotspotImgView[i].hidden = YES;	
					hotspotDisplayed[i] = false;
					hotspotUsed[i] = false;			
				}
			}
			
			//etat action
			if( mEtatAction != 0 )
				[self resetAction:0 coordY:0];
			
			//si pas dans hotspot, si pas animation début du jeu, si pas fin niveau, si pas fin jeu, alors
			//action deplacement rapide ou action by touch long
			if( !iconsDisplayed && strcmp(niveau," ") != 0 && !bProtecOFF && !bPopup && !bLoadingLevel && bLvLLoaded && !bSimulation  && !bMenuIngame && !bHintMenuActive ) 
			{
				//ON NETTOIE
				for( int i = 0; i < _HotspotObj.size(); ++i )
				{
					alphaDisplayed[i] = false;						
					hotspotImgView[i].hidden = YES;
					hotspotDisplayed[i] = false;
					hotspotUsed[i] = false;				
				}
				[self clearIcons];
				//END
				
				//touch time End
				endD = [NSDate date];
				
				//touch rapide (deplacement)
				if( !bMouseModeOn && [endD compare:debD] == NSOrderedAscending )
				{
					[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
									[NSNumber numberWithFloat:point.x], @"x",
									[NSNumber numberWithFloat:point.y], @"y",nil]];	
					
					[NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(affecAction) userInfo:nil repeats:NO];
				}
				else //touch long (selection actions)
				{
					bStepIconOn = true;
					
					stepPosX = point.x;
					stepPosY = point.y;
					
					if( rondV == nil )
					{
						rondV = [[UIImageView alloc] initWithFrame:CGRectMake(point.x*480/320, point.y*320/200, 10, 10)];		
						UIImage* rond = [[UIImage imageNamed:@"rond bleu.png"] retain];		
						[rondV setImage:rond];		
						[rond release];		
						[self addSubview:rondV];
					}
					else
					{
						rondV.frame = CGRectMake(point.x*480/320, point.y*320/200, 10, 10);
						rondV.hidden = NO;
					}
					
					//compute the position of the icons and display them
					[self iconsPositionCompute:point.x coordY:point.y];
					
					// hide cursor mouse
					bMouseCursorActive = false;
					
					if( cursorGobliiin != nil && cursorGobliiin.superview != nil )
					{
						cursorGobliiin.hidden = YES;
					}
				}
			}	
			else
			{				
				bStepIconOn = false;
				
				// hide cursor mouse
				bMouseCursorActive = false;
				
				if( cursorGobliiin != nil && cursorGobliiin.superview != nil )
				{
					cursorGobliiin.hidden = YES;
				}
			}
			iconsDisplayed = false;
		}
	}
}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event
{
	//NSLog(@"iPhoneView touchesCancelled");	
}

- (void)resetAction:(float)xTouch coordY:(float)yTouch
{	
	//NSLog(@"iPhoneView resetAction");
	switch( mEtatAction )
	{
		case 0:
			break;
		case 3:
		{
			if( bGobPicker )
			{
				[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonRightDown], @"type",
								[NSNumber numberWithFloat:xTouch], @"x",
								[NSNumber numberWithFloat:yTouch], @"y",nil]];
				
				[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kTempo], @"type",
								[NSNumber numberWithFloat:xTouch], @"x",
								[NSNumber numberWithFloat:yTouch], @"y",nil]];
				
				[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonRightDown], @"type",
								[NSNumber numberWithFloat:xTouch], @"x",
								[NSNumber numberWithFloat:yTouch], @"y",nil]];		
				
				[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kTempo], @"type",
								[NSNumber numberWithFloat:xTouch], @"x",
								[NSNumber numberWithFloat:yTouch], @"y",nil]];
			}
			else
			{
				[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonRightDown], @"type",
								[NSNumber numberWithFloat:xTouch], @"x",
								[NSNumber numberWithFloat:yTouch], @"y",nil]];
				
				[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kTempo], @"type",
								[NSNumber numberWithFloat:xTouch], @"x",
								[NSNumber numberWithFloat:yTouch], @"y",nil]];
			}
		}
			break;
		case 4:
		{
			[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonRightDown], @"type",
							[NSNumber numberWithFloat:xTouch], @"x",
							[NSNumber numberWithFloat:yTouch], @"y",nil]];
			
			[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kTempo], @"type",
							[NSNumber numberWithFloat:xTouch], @"x",
							[NSNumber numberWithFloat:yTouch], @"y",nil]];
		}
			break;
	}
	mEtatAction = 0;	
}

-(void)affecAction
{
	//NSLog(@"ACTION = %d",gg->_goblin->_gobAction);
	mEtatAction = gg->_goblin->_gobAction;
}

- (void)pickObject:(float)xTouch coordY:(float)yTouch
{	
	[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonRightDown], @"type",
					[NSNumber numberWithFloat:xTouch], @"x",
					[NSNumber numberWithFloat:yTouch], @"y",nil]];
	
	[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kTempo], @"type",
					[NSNumber numberWithFloat:xTouch], @"x",
					[NSNumber numberWithFloat:yTouch], @"y",nil]];
	
	[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonRightLeft], @"type",
					[NSNumber numberWithFloat:xTouch], @"x",
					[NSNumber numberWithFloat:yTouch], @"y",nil]];
	
	[NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(affecAction) userInfo:nil repeats:NO];	
}

- (void)clearButtons
{	
	// hide cursor mouse
	bMouseCursorActive = true;	
	
	if( bMouseModeOn )
	{
		//[self cursorCurrentGobliiin:((OSystem_IPHONE*)g_system)->_mouseX coordY:((OSystem_IPHONE*)g_system)->_mouseY];
		cursorGobliiin.frame = CGRectMake(((OSystem_IPHONE*)g_system)->_mouseX *480/320,((OSystem_IPHONE*)g_system)->_mouseY *320/200, 40, 40);
		cursorGobliiin.hidden = NO;
	}
	
	//-----------------------
	//we re quitting this mode
	iconsDisplayed = false;
	useIcon.hidden = YES;
	pickIcon.hidden = YES;
	stepIcon.hidden = YES;
	cancelIcon.hidden = YES;
	bStepIconOn = false;
	
	rondV.hidden = YES;
	
	//we clear the view, and these are set to false
	for( int i = 0; i < 20; ++i )
	{
		alphaDisplayed[i] = false;		
		hotspotImgView[i].hidden = YES;
		hotspotDisplayed[i] = false;
		hotspotUsed[i] = false;
	}
	//------------------------
	//END
}

//anim alpha
-(void) beginAlpha:(UIButton*)i
{
	if( buttonFade )
	{
		i.alpha = 1;
		
		[UIView beginAnimations:nil context:(void*)i]; 
		{
			[UIView setAnimationDuration:0.25];
			[UIView setAnimationDelegate:self];
			[UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
			[UIView setAnimationDidStopSelector:@selector(alphaFinished:finished:context:)];
			i.alpha = 0.25;
		}
		[UIView commitAnimations];
	}
}

- (void)alphaFinished:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context
{	
	if( buttonFade )
	{
		UIButton* i = (UIButton*)context;
		i.alpha = 0.25;
		
		[UIView beginAnimations:nil context:(void*)i]; 
		{
			[UIView setAnimationDuration:0.25];
			[UIView setAnimationDelegate:self];
			[UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
			[UIView setAnimationDidStopSelector:@selector(alphaBeginFinished:finished:context:)];
			i.alpha = 1.0;
		}
		[UIView commitAnimations];
	}
}

- (void)alphaBeginFinished:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context
{	
	if( buttonFade )
	{
		UIButton* i = (UIButton*)context;
		[self beginAlpha:i];
	}
}

-(void)doInteractOk
{
	[self setUserInteractionEnabled:YES];
}

-(void)doClear
{
	buttonFade = false;
	useIcon.alpha = 1;
	cancelIcon.alpha = 1;
	stepIcon.alpha = 1;
	pickIcon.alpha = 1;
	
	//clear all attributes
	[self clearButtons];
	
	if( !bCancelAction )
		[NSTimer scheduledTimerWithTimeInterval:0.4 target:self selector:@selector(doInteractOk) userInfo:nil repeats:NO];
	else
	{
		bCancelAction = false;
		[NSTimer scheduledTimerWithTimeInterval:0 target:self selector:@selector(doInteractOk) userInfo:nil repeats:NO];		
	}
}

-(void)doUse
{
	//SIMULATION DU CLICK DROIT POUR SIMULER ACTION UTILISER	
	int ind = 0;
	for( int i = 0; i < 20; ++i )
		if( hotspotUsed[i] )
			ind = i;
	
	float xTouch;
	float yTouch;
	
	if( !bStepIconOn ) //si choix actions not displayed
	{				
		xTouch = _HotspotObj[ind]->left + (_HotspotObj[ind]->right - _HotspotObj[ind]->left)/2 - 5;
		yTouch = _HotspotObj[ind]->top + (_HotspotObj[ind]->bottom - _HotspotObj[ind]->top)/2 - 5;
		
		//A VERIF LVL 6 ONLY ???
		if( idLvLCourant == 6 )
		{
			if( _HotspotObj[ind]->top == 0 )
			{
				xTouch = _HotspotObj[ind]->left + (_HotspotObj[ind]->right - _HotspotObj[ind]->left)/2;
				yTouch = _HotspotObj[ind]->top + (_HotspotObj[ind]->bottom - _HotspotObj[ind]->top)/2 + 20; //HACK
			}
		}
		
		//PB LVL 21
		if( idLvLCourant == 21 )
		{
			if( ind == 1 )
			{
				xTouch = _HotspotObj[ind]->left + (_HotspotObj[ind]->right - _HotspotObj[ind]->left)/2;
				yTouch = _HotspotObj[ind]->top + (_HotspotObj[ind]->bottom - _HotspotObj[ind]->top)/2 - 15;
			}
		}
		
		if( yTouch < 0 )
			yTouch = _HotspotObj[ind]->top;
	}
	else
	{
		xTouch = stepPosX;
		yTouch = stepPosY;
	}	
	g_system->warpMouse( xTouch, yTouch );
	
	//utilisation action "use"
	[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonRightLeft], @"type",
					[NSNumber numberWithFloat:xTouch], @"x",
					[NSNumber numberWithFloat:yTouch], @"y",nil]];
	
	[NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(affecAction) userInfo:nil repeats:NO];
}

- (IBAction) useIconClicked:(id)sender
{
	//NSLog(@"iPhoneView useIconClicked");
	[self setUserInteractionEnabled:NO];
	
	buttonFade = true;
	[self beginAlpha:useIcon];
	
	[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(doUse) userInfo:nil repeats:NO];
	
	[NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(doClear) userInfo:nil repeats:NO];
}

-(void) doPick
{
	//SIMULATION DU CLICK DROIT POUR SIMULER ACTION PRENDRE	
	int ind = 0;
	for( int i = 0; i < 20; ++i )
		if( hotspotUsed[i] )
			ind = i;
	
	float xTouch;
	float yTouch;
	
	if( !bStepIconOn )
	{						
		xTouch = _HotspotObj[ind]->left + (_HotspotObj[ind]->right - _HotspotObj[ind]->left)/2 - 10;
		yTouch = _HotspotObj[ind]->top + (_HotspotObj[ind]->bottom - _HotspotObj[ind]->top)/2 - 10;
		
		if( idLvLCourant == 17 )
		{
			yTouch = _HotspotObj[ind]->top + (_HotspotObj[ind]->bottom - _HotspotObj[ind]->top)/2 - 25;
			
			if( ind == 4 )
			{
				xTouch = _HotspotObj[ind]->left;
				yTouch = _HotspotObj[ind]->top + 5;
			}
			else if( ind == 5 )
			{
				yTouch = _HotspotObj[ind]->top + (_HotspotObj[ind]->bottom - _HotspotObj[ind]->top)/2 - 15;
			}
		}
		
		if( yTouch < 0 )
			yTouch = _HotspotObj[ind]->top;
	}
	else
	{
		xTouch = stepPosX;
		yTouch = stepPosY;
	}
	g_system->warpMouse( xTouch, yTouch );
	
	if( bGobPicker )
	{
		[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kTempo], @"type",
						[NSNumber numberWithFloat:xTouch], @"x",[NSNumber numberWithFloat:yTouch], @"y",nil]];
		
		//pick object by picker
		[self pickObject:xTouch coordY:yTouch];
	}
}

- (IBAction) pickIconClicked:(id)sender
{
	//NSLog(@"iPhoneView pickIconClicked");
	
	[self setUserInteractionEnabled:NO];
	
	buttonFade = true;
	[self beginAlpha:pickIcon];
	
	[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(doPick) userInfo:nil repeats:NO];
	
	[NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(doClear) userInfo:nil repeats:NO];
}

-(void) doStep
{
	//SIMULATION 
	int ind = 0;
	for( int i = 0; i < 20; ++i )
		if( hotspotUsed[i] )
			ind = i;
	
	float xTouch;
	float yTouch;
	
	if( !bStepIconOn )
	{		
		xTouch = _HotspotObj[ind]->left + (_HotspotObj[ind]->right - _HotspotObj[ind]->left)/2;
		yTouch = _HotspotObj[ind]->top + (_HotspotObj[ind]->bottom - _HotspotObj[ind]->top)/2;
	}
	else
	{
		xTouch = stepPosX;
		yTouch = stepPosY;
	}
	g_system->warpMouse( xTouch, yTouch );
	
	//move the goblin
	[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
					[NSNumber numberWithFloat:xTouch], @"x",
					[NSNumber numberWithFloat:yTouch], @"y",nil]];	
	
	[NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(affecAction) userInfo:nil repeats:NO];	
}

- (IBAction) stepIconClicked:(id)sender
{
	//NSLog(@"iPhoneView stepIconClicked");
	
	[self setUserInteractionEnabled:NO];
	
	buttonFade = true;
	[self beginAlpha:stepIcon];
	
	[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(doStep) userInfo:nil repeats:NO];
	
	[NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(doClear) userInfo:nil repeats:NO];
}

- (IBAction) cancelIconClicked:(id)sender
{
	//NSLog(@"iPhoneView cancelIconClicked");
	
	buttonFade = true;
	[self beginAlpha:cancelIcon];
	
	[self setUserInteractionEnabled:NO];
	
	bCancelAction = true;
	
	[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(doClear) userInfo:nil repeats:NO];
}

-(void)resetPosCursor
{
	((OSystem_IPHONE*)g_system)->_mouseX = xPosCur;
	((OSystem_IPHONE*)g_system)->_mouseY = yPosCur;
	reset = false;
}

-(IBAction) choiceButtonClicked:(id)sender
{
	//NSLog(@"iPhoneViewController choiceButtonClicked");	
	if( !bTutoOn )
	{
		if( !bMenuIngame && !bHintMenuActive && !reset )
		{
			reset = true;
			
			//on nettoie pour changer // TO DO - changer en fonction du goblin
			[self clearButtons];
			
			//update hotspot
			gg = (Gob::GobEngine*)g_engine;	
			gg->_goblin->updateHotspot();
			
			xPosCur = ((OSystem_IPHONE*)g_system)->_mouseX;
			yPosCur = ((OSystem_IPHONE*)g_system)->_mouseY;
			
			if( numGobliiin == 2 )
				numGobliiin = 0;
			else
				numGobliiin++;
			
			if( bMouseModeOn )
				[self cursorCurrentGobliiin:((OSystem_IPHONE*)g_system)->_mouseX coordY:((OSystem_IPHONE*)g_system)->_mouseY];
			
			float xTouch = 170;
			float yTouch = 180;
			g_system->warpMouse( xTouch, yTouch );
			
			//move the goblin
			[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
							[NSNumber numberWithFloat:xTouch], @"x",
							[NSNumber numberWithFloat:yTouch], @"y",nil]];	
			
			if( bMouseModeOn )
				[NSTimer scheduledTimerWithTimeInterval:0.4 target:self selector:@selector(resetPosCursor) userInfo:nil repeats:NO];
			else 
				reset = false;
		}
	}
}

@end

//---C functions
int lvlStringToInt( const char* s )
{
	int num(0);
	
	for( int i = 0; i < 22; ++i )
	{
		if( strcmp( m_lvl[i], s ) == 0 )
			num = i;
	}
	return num+1;
}

bool getLocalMouseCoords(CGPoint *point)
{
	//printf("point %d;%d - screenRect %d;%d w:%d h:%d\n",point->x, point->y, _screenRect.origin.x,  _screenRect.origin.y, _screenRect.size.width, _screenRect.size.height);
	if (point->x < _screenRect.origin.x || point->x >= _screenRect.origin.x + _screenRect.size.width ||
		point->y < _screenRect.origin.y || point->y >= _screenRect.origin.y + _screenRect.size.height)
	{
		return false;
	}
	
	point->x = (point->x - _screenRect.origin.x) / _screenRect.size.width;
	point->y = (point->y - _screenRect.origin.y) / _screenRect.size.height;
	
	return true;
}

uint getSizeNextPOT(uint size) 
{
    if ((size & (size - 1)) || !size)
	{
        int log = 0;
		
        while (size >>= 1)
            ++log;
		
        size = (2 << log);
    }
	
    return size;
}
//----

void iPhone_updateScreen()
{
	if (!_needsScreenUpdate) 
	{
		_needsScreenUpdate = 1;
		[sharedInstance performSelectorOnMainThread:@selector(updateSurface) withObject:nil waitUntilDone: NO];
	}
}

void iPhone_updateScreenRect(unsigned short* screen, int x1, int y1, int x2, int y2) 
{
	//[_lock lock];

	int y;
	for (y = y1; y < y2; ++y) 
	{
		memcpy(&_textureBuffer[(y * _textureWidth + x1 )* 2], &screen[y * _width + x1], (x2 - x1) * 2);
	}

	//[_lock unlock];
}


void iPhone_initSurface(int width, int height)
{
	_width = width;
	_height = height;
	[sharedInstance performSelectorOnMainThread:@selector(initSurface) withObject:nil waitUntilDone: YES];
}

bool iPhone_fetchEvent(int *outEvent, float *outX, float *outY)
{
	id event = [sharedInstance getEvent];
	if (event == nil) 
	{
		return false;
	}

	id type = [event objectForKey:@"type"];

	if (type == nil)
	{
		//printf("fetchEvent says: No type!\n");
		return false;
	}

	*outEvent = [type intValue];
	*outX = [[event objectForKey:@"x"] floatValue];
	*outY = [[event objectForKey:@"y"] floatValue];
	return true;
}

const char* iPhone_getDocumentsDir()
{
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	return [documentsDirectory UTF8String];
}

const char* iPhone_getBundleDir() 
{
	NSString *bundleDirectory = [[NSBundle mainBundle] bundlePath];	
	return [bundleDirectory UTF8String];
}

